<?php
namespace Util;
use Zeedhi\Framework\DependencyInjection\InstanceManager;
use Zeedhi\Framework\DTO\Response\Error;
use Exception\CustomException;
use Doctrine\DBAL\DriverManager;
use Util\Instance\GeneralProvider;

class Utilities {
    const GERAL_ENTITY_DIR = '\Teknisa\Model\Geral\Entities\\';
    const TEKNISA_ENTITY_DIR = '\Teknisa\Model\Teknisa\Entities\\';

    /**
     * @param array $filters
     * @param $nameFilter
     * @param bool $remove
     * @return null
     */
    public static function getOwner(){
        $connectionParams = InstanceManager::getInstance()->getParameter('connection_params_reference');
        return isset($connectionParams['sessionMode']) ? $connectionParams['dbname'] : $connectionParams['user'];
    }

    public static function getFileServerFilesUrlParameter() {
        return InstanceManager::getInstance()->getParameter('fileServerFilesUrl');
    }

    public static function getTempLogPathParameter() {
        return InstanceManager::getInstance()->getParameter('tempLogPath');
    }

    /**
     * @param CustomException $e
     * @return Error
     */
    public static function getErrorObjectFromException(CustomException $e) {
        $error = array (
            'CODE'   => $e->getMessage(),
            'PARAMS' => $e->getParams()
        );
        return new Error(json_encode($error), $e->getCode(), $e->getTraceAsString());
    }

    public static function getFilterTableColumn($filters, $tableColumn)
    {
        $filterLabel = [];
        foreach ($filters as $filter) {
            if ($filter['columnName'] === $tableColumn) {
                $filterLabel =  [$filter];
            }
        }
        return $filterLabel;
    }

    public static function getValueInFilterDataSourceByName(array &$filters, $nameFilter, $remove = false){
        foreach ($filters as $key => $filter) {
            $name = $filter['columnName'];
            $values = $filter['value'];
            if($name == $nameFilter ){
                if($remove){
                    unset($filters[$key]);
                }
                return $values;
            }
        }
        return null;
    }

    // self-explanatory
    public static function createQueryFiltered($filters, $tableAlias = null)
    {
        $filterSQL = '';
        foreach ($filters as $filter) {
            $name = $filter['columnName'];
            $operator = $filter['operator'] == "LIKE_I" ? "LIKE" : $filter['operator'];
            $values = $filter['value'];
            if($name == 'NRLICENCAUSO'){
                $operator = 'LIKE';
            }
            //(!is_null($values) && $operator == '=') ->Gambi to fix framework bugs :(
            if ((($name != '*' && !empty($values)) || $values == 0) && !(empty($values) && $operator == '=') && !is_null($values)) {
                if ($operator == 'BETWEEN') {
                    if ($values[0] && $values[1]) {
                        $values = "TO_DATE( '$values[0] 00:00:00' ,'DD/MM/YYYY HH24:MI:SS')
                                AND TO_DATE( '$values[1] 23:59:59' ,'DD/MM/YYYY HH24:MI:SS')";
                    } else if ($values[0]) {
                        $operator = '>';
                        $values = "TO_DATE( '$values[0] 00:00:00' ,'DD/MM/YYYY HH24:MI:SS')";
                    } else if ($values[1]) {
                        $operator = '<';
                        $values = "TO_DATE( '$values[1] 00:00:00' ,'DD/MM/YYYY HH24:MI:SS')";
                    } else {
                        $values = "";
                    }

                    if ($values || ($values && ($values[0] || $values[1]))) {
                        if (!empty($tableAlias)) {
                            $filterSQL .= "AND $tableAlias.$name $operator $values";
                        } else {
                            $filterSQL .= "AND $name $operator $values";
                        }
                    }
                } else {
                    if (is_array($values) && $operator != 'MAPPED_LIKE_ALL') {
                        $stringValues = "('" . implode($values, "','") . "')";
                        $values = $stringValues;
                    }
                    if ($operator == 'LIKE' || $operator == 'LIKE_I') {
                        $operator = 'LIKE';
                        if (!empty($tableAlias)) {
                            $values = "UPPER('" . $values . "')";
                            $name = "UPPER($tableAlias.$name)";
                        } else {
                            $values = "UPPER('" . $values . "')";
                            $name = "UPPER($name)";
                        }
                    }
                    if ($operator == 'LIKE_ALL') {
                        if (!empty($tableAlias)) {
                            $values = "UPPER('" . $values . "')";
                        } else {
                            $values = "UPPER('" . $values . "')";
                        }

                        $columnList = explode('|', $name);
                        $likes = array();
                        foreach ($columnList as $column) {
                            $likes[] = "UPPER($column) LIKE $values";
                        }
                        $filterLikeAll = implode($likes, ' OR ');
                        $filterSQL .= " AND ( $filterLikeAll )";
                    } else if ($operator == 'LIKE') {
                        $filterSQL .= " AND $name $operator $values ";
                    } else if ($operator == 'IS_NULL') {
                        if (!empty($tableAlias)) {
                            $filterSQL .= " AND $tableAlias.$name IS $values ";
                        } else {
                            $filterSQL .= " AND $name IS $values ";
                        }
                    } else if ($operator == '='){
                        if(!empty($tableAlias)){
                            $filterSQL .= " AND $tableAlias.$name $operator '$values' ";
                        } else {
                            $filterSQL .= " AND $name $operator '$values' ";
                        }
                    } else if ($operator == 'MAPPED_LIKE_ALL') {
                        foreach($values[0] as $index => $mappedLikeAllValues) {
                            if($index == '_ALL') {
                                $values = "UPPER('" . $mappedLikeAllValues . "')";

                                $columnList = explode('|', explode('#', $name)[0]);
                                $likes = array();
                                foreach ($columnList as $column) {
                                    if (empty($tableAlias)) {
                                        $likes[] = "UPPER($column) LIKE $values";
                                    } else {
                                        $likes[] = "UPPER($tableAlias.$column) LIKE $values";
                                    }
                                }
                                $filterLikeAll = implode($likes, ' OR ');
                                $filterSQL .= " AND ( $filterLikeAll )";
                            } else {
                                $stringValues = "('" . implode($mappedLikeAllValues, "','") . "')";
                                if (!empty($tableAlias)) {
                                    $filterSQL .= " AND $tableAlias.$index IN $stringValues ";
                                } else {
                                    $filterSQL .= " AND $index IN $stringValues ";
                                }
                            }
                        }
                    } else {
                        if (!empty($tableAlias)) {
                            $filterSQL .= " AND $tableAlias.$name $operator $values ";
                        } else {
                            $filterSQL .= " AND $name $operator $values ";
                        }
                    }
                }
            }
        }
        return $filterSQL;
    }

    public static function getPaginatedQuery($query, $limit, $offset) {
        if ($limit > 0) {
            $column = '*';
            if ($offset > 0) {
                $min = $offset + 1;
                $query = 'SELECT * FROM (SELECT a.' . $column . ', rownum AS doctrine_rownum FROM (' .
                    $query .
                    ') a WHERE rownum <= ' . $limit . ') WHERE doctrine_rownum >= ' . $min;
            } else {
                $query = 'SELECT a.' . $column . ' FROM (' . $query . ') a WHERE ROWNUM <= ' . $limit;
            }
        }
        return $query;
    }

    /**
     * @return \Doctrine\DBAL\Connection
     * @throws \Doctrine\DBAL\DBALException
     */
    public static function getReferenceConnection( $connectionType = 'EXEC') {
        $connectionParamsName = ($connectionType == "EXEC") ? 'connection_params_reference' : 'connection_params_reference_parse';
        $connectionParams = InstanceManager::getInstance()->getParameter($connectionParamsName);
        $configuration    = GeneralProvider::getEntityMetaDataConfig();
        $eventManager     = GeneralProvider::getEventManager();
        return DriverManager::getConnection($connectionParams, $configuration, $eventManager);
    }

    public static function getRemoteFileSize($url) {
        $data = get_headers($url, true);
        if (isset($data['Content-Length'])) {
            return (int)$data['Content-Length'];
        }
        return null;
    }

    public static function downloadAndRemoveFile($fileDir, $fileName, $fileType = 'text/plain', $isRemoveFile = true) {
        if(strpos($fileDir, 'http') === false) {
            $fileSize = filesize($fileDir);
        } else {
            $fileSize = self::getRemoteFileSize($fileDir);
        }
        header('Accept-Ranges: none');
        header("Content-Type: " . $fileType . "; charset=ISO-8859-1");
        header("Content-Length: " . $fileSize);
        header("Content-Disposition: attachment; filename=" . $fileName);
        readfile($fileDir);

        if($isRemoveFile) {
            unlink($fileDir);
        }
        exit;
    }

    public static function getStructureUpdateIgnoredErrorsParameter() {
        return InstanceManager::getInstance()->getParameter('structureUpdateIgnoredErrors');
    }
    public static function getArrayByString($stringValue, $delimiter = ',', $int = false)
    {
        if (empty($stringValue)) {
            $value = array();
        } else {
            if ($int) {
                $value = array_map('intval', explode($delimiter, $stringValue));
            } else {
                $value = explode($delimiter, $stringValue);
            }
        }
        return $value;
    }
     public static function setSelectMultiInQuery($stringValue, $int = false, $delimiter = ',',$uniqueArray = false)
    {
        $array = self::getArrayByString($stringValue, $delimiter, $int);
        if($uniqueArray){
            $array = array_unique($array);
        }
        return $array;
    }
    
}