
// FILE: assets/js/Util/Utilities.js
function Utilities(ScreenService, templateManager, MessageService, ZhPreferences, ZhFilterPreferences, Query, ZHPromise,requestDataSourceEngine) {
    var self = this;

    this.SCRIPT_EXECUTION_SUCCESS = 'SCRIPT_SUCCESS';
    this.SCRIPT_EXECUTION_FAIL    = 'SCRIPT_FAIL';
    this.LOG_COMMAND_SEPARATOR    = '\n----------------------------------------------------------------------------------------------------------------------\n\n';

    this.setFilterInit = function (widget) {
        widget.mtAppliedFilter = true;
    };

    this.isAppliedFilter = function(widget){
        return !!widget.mtAppliedFilter;
    };

    this.getCurrentDateWithTime = function () {
        return moment().format('DD/MM/YYYY HH:mm:ss');
    };


    this.genericDeleteEvent = function (widget){
        return widget.dataSource.remove([widget.currentRow]).then(function(data){
            return ScreenService.closeSwipe().then(function(){
                return widget.reload();
            });
        });
    };

    this.getCurrentDateInMilliseconds = function(){
        return new Date().getTime();
    };

    this.getCurrentFormattedDateForAttachment = function(){
        return moment().format('YYYY-MM-DDTHH:MM:SSZ');
    };

    this.transliterateString = function(text) {
        var transliterationTable = {
            "á" : "a",  "Á" : "A",  "à" : "a",  "À" : "A",  "a" : "a",
            "A" : "A",  "â" : "a",  "Â" : "A",  "å" : "a",  "Å" : "A",
            "ã" : "a",  "Ã" : "A",  "ä" : "ae", "Ä" : "AE", "æ" : "ae",
            "Æ" : "AE", "c" : "c",  "C" : "C",  "ç" : "c",  "Ç" : "C",
            "d" : "d",  "D" : "D",  "Ð" : "Dh", "ð" : "dh", "é" : "e",
            "É" : "E",  "è" : "e",  "È" : "E",  "e" : "e",  "E" : "E",
            "ê" : "e",  "Ê" : "E",  "ë" : "e",  "Ë" : "E",  "ƒ" : "F",
            "g" : "g",  "G" : "G",  "h" : "h",  "H" : "H",  "í" : "i",
            "Í" : "I",  "ì" : "i",  "Ì" : "I",  "î" : "i",  "Î" : "I",
            "ï" : "i",  "Ï" : "I",  "i" : "i",  "I" : "I",  "j" : "j",
            "J" : "J",  "k" : "k",  "K" : "K",  "l" : "l",  "L" : "L",
            "n" : "n",  "N" : "N",  "ñ" : "n",  "Ñ" : "N",  "ó" : "o",
            "Ó" : "O",  "ò" : "o",  "Ò" : "O",  "ô" : "o",  "Ô" : "O",
            "o" : "o",  "O" : "O",  "õ" : "o",  "Õ" : "O",  "ø" : "oe",
            "Ø" : "OE", "ö" : "oe", "Ö" : "OE", "r" : "r",  "R" : "R",
            "s" : "s",  "S" : "S",  "š" : "s",  "Š" : "S",  "ß" : "SS",
            "t" : "t",  "T" : "T",  "ú" : "u",  "Ú" : "U",  "ù" : "u",
            "Ù" : "U",  "u" : "u",  "U" : "U",  "û" : "u",  "Û" : "U",
            "ü" : "ue", "Ü" : "UE", "w" : "w",  "W" : "W",  "ý" : "y",
            "Ý" : "Y",  "y" : "y",  "Y" : "Y",  "ÿ" : "y",  "Ÿ" : "Y",
            "z" : "z",  "Z" : "Z",  "ž" : "z",  "Ž" : "Z",  "þ" : "th",
            "Þ" : "Th", "µ" : "u"
        };
        var newText = "";
        if(text){
            for(var i = 0; i < text.length; i++){
                newText += transliterationTable[text[i]] || text[i];
            }
        }
        return newText;
    };

    this.b64Encode = function(str) {
        // first we use encodeURIComponent to get percent-encoded UTF-8,
        // then we convert the percent encodings into raw bytes which
        // can be fed into btoa.
        return btoa(encodeURIComponent(str).replace(/%([0-9A-F]{2})/g,
                function toSolidBytes(match, p1) {
                    return String.fromCharCode('0x' + p1);
                })
        );
    };

    this.byteLength = function(str) {
        var s = str.length;
        for (var i=str.length-1; i>=0; i--) {
            var code = str.charCodeAt(i);
            if (code > 0x7f && code <= 0x7ff) s++;
            else if (code > 0x7ff && code <= 0xffff) s+=2;
            if (code >= 0xDC00 && code <= 0xDFFF) i--;
        }
        return s;
    };

    this.showErrorMessage = function(error) {
        var message = error;
        if(typeof error === 'object') {
            if(error.message) {
                message = error.message;
            } else {
                message = error.data.error;
            }
        }
        ScreenService.showMessage(message, 'error', 'error');
    };

    /*fix bug SelectAutocomplete */
    this.initializeFieldSelectAutocomplete = function (widgetName, field) {
        var widget = templateManager.container.getWidget(widgetName);
        widget.getField(field).initializeField();
    };

    this.isJson = function(str) {
        var result = null;

        try {
            result = JSON.parse(str);
        } catch (e) { }

        return result;
    };


    this.showMessageErrorOfBackEnd = function(response) {
        return MessageService.getMessageByCode('UNEXPECTED_ERROR').then(function(defaultMessage) {
            if (response && response.data && response.data.error) {
                var error = response.data.error;
                if (self.isJson(error)) {
                    error = JSON.parse(error);
                    var params = error.PARAMS || null;
                    var messageCode = error.CODE || 'UNEXPECTED_ERROR';
                    return MessageService.getMessageByCode(messageCode).then(function(message) {
                        return ScreenService.showMessage(message, 'ERROR', null, params);
                    }).catch(function() {
                        return ScreenService.showMessage(messageCode, 'ERROR', null, params);
                    });
                } else {
                    var message = Util.isEmptyOrBlank(error) ? defaultMessage : error.split("Exception:").pop();
                    return ScreenService.showMessage(message, 'ERROR');
                }
            } else {
                return ScreenService.showMessage(defaultMessage, 'ERROR');
            }
        });
    };

    this.applyDefaultFilterOnEnter = function(widget, isApplyFilter) {
        var time = 500;
        return setTimeout(function() {
            self.applyDefaultFilter(widget, isApplyFilter);
        }, time);
    };

    this.setDefaultFilterOnEnter = function(widget, isApplyFilter) {
        self.applyDefaultFilterOnEnter(widget, isApplyFilter);
    };

    this.applyDefaultFilter = function(widget, isApplyFilter){
        if (widget.inListMode()) {
            if (widget.mtAppliedFilter) {
                widget.reload();
            } else if (widget.currentRowDefaultFilter) {
                self.startFilter(widget, widget.currentRowDefaultFilter, isApplyFilter);
            } else {
                self.startFilter(widget, widget.currentRow, isApplyFilter);
            }
        }
    };

    this.showMessageBackEndTrue = function(response) {
        return self.showMessageErrorOfBackEnd(response);
    };

    this.startFilter = function(widget, currentRow, isApplyOriginalFilter){
        ZhPreferences.getDefaultWidgetPreferences(widget.id).then(function(result) {
            if (result && result.DEFAULT_FILTER) {
                ZhPreferences.getPreferences('filter', Query.build().where('WIDGET_ID').equals(widget.id)).then(function(filters) {
                    var defaultFilter = filters.filter(function(filter) {
                        return filter.ID == result.DEFAULT_FILTER;
                    })[0];
                    var filter = {
                        ID: defaultFilter.LABEL_TO_PREFERENCE || 'EDITED',
                        FILTER_ROW: JSON.parse(defaultFilter.FILTER_ROW)
                    };
                    ZhFilterPreferences.applyFilter(widget, filter).then(function() {
                        widget.appliedFilter = widget.appliedFilter || filter;
                        widget.appliedFilter.ID = result.DEFAULT_FILTER;
                        if(defaultFilter.LABEL_TO_PREFERENCE) {
                            widget._filterInfo[0].value = defaultFilter.LABEL_TO_PREFERENCE;
                        }
                    });
                });
            } else if(isApplyOriginalFilter) {
                ZhFilterPreferences.applyOriginalFilter(widget);
            } else {
                ZhFilterPreferences.openFilterWidget(widget, currentRow);
            }
        });
    };

    this.b64Decode = function(str) {
        // Going backwards: from bytestream, to percent-encoding, to original string.
        try {
            return decodeURIComponent(atob(str).split('').map(function (c) {
                return '%' + ('00' + c.charCodeAt(0).toString(16)).slice(-2);
            }).join(''));
        } catch (e) {
            throw e;
        }
    };

    this.b64Encode = function(str) {
        // first we use encodeURIComponent to get percent-encoded UTF-8,
        // then we convert the percent encodings into raw bytes which
        // can be fed into btoa.
        return btoa(encodeURIComponent(str).replace(/%([0-9A-F]{2})/g,
                function toSolidBytes(match, p1) {
                    return String.fromCharCode('0x' + p1);
                })
        );
    };

    this.doSyncMtFeatParametersForScript = function(functions, currentRow, scripts, connType, logField, defer) {
        return functions.reduce(function(lastPromise, nextFunction, index) {
            return lastPromise.then(function() {
                return nextFunction(currentRow, scripts[index], connType, logField, defer);
            });
        }, ZHPromise.when());
    };
    
    this.mergeDataSetPagnation = function (field, filterMerge, valueFieldIndex, notFilter) {
        if(filterMerge){
            var filter = [];
            if(field.dataSourceFilter && !notFilter){
                filter = filter.concat(field.dataSourceFilter,filterMerge);
            }else{
                filter = filterMerge;
            }
            return requestDataSourceEngine.getDataSource(field.dataSource.name, filter).then(function (response) {
                var fieldValues = response.dataset[field.dataSource.name.split('/')[1]];
                fieldValues.forEach(function (fieldvalue) {
                    var validate = field.dataSource.data.filter(function (data) {
                        return data[valueFieldIndex] == fieldvalue[valueFieldIndex];
                    });
                    if (!validate.length) {
                        field.dataSource.data.push(fieldvalue);
                    }
                });
                templateManager.updateTemplate();
            });
        }
    };
    
    this.bugPaginationOnSelectMultiplyOnModeView = function(field){
        var valueFieldIndex = field.valueField;
        var filter = [
            {
                "operator": "IN",
                "name": valueFieldIndex,
                "value": field.value() || [0]
            }
        ];
        return this.mergeDataSetPagnation(field, filter, valueFieldIndex);
    };
    
    this.setValueInSelectField = function(field,value){
        field.widget.currentRow[field.outData[field.valueField]] = value;
        if(field.hasPagination !== false){
            this.bugPaginationOnSelectOnModeView(field,value).then(function(){
                self.setOutData(field, value);
            });
        }else {
            this.setOutData(field, value);
        }
    };
}

Configuration(function(ContextRegister){
    ContextRegister.register('Utilities', Utilities);
});

// FILE: assets/js/Util/WebSocketClient.js
function WebSocketClient(GitQueue, ZHPromise) {

    this.connectInServer = function () {
        GitQueue.connect(wsServer);
    };

    this.loginInServer = function (id) {
        return GitQueue.loginBrowser(id);
    };

    this.createFile = function (path, fileName, fileContents, room) {
        var defer = ZHPromise.defer();
        var reject = function (error) {
            defer.reject(error);
        };
        var resolve = function () {
            defer.resolve();
        };
        (room || GitQueue).createPath(path)
            .error(reject)
            .then(function () {
                (room || GitQueue).createFile(path, fileName, fileContents)
                    .error(reject)
                    .then(resolve);
            });

        return defer.promise;
    };

    this.wget = function (path, url, fileName, room) {
        var defer = ZHPromise.defer();
        var reject = function (error) {
            defer.reject(error);
        };
        var resolve = function () {
            defer.resolve();
        };

        (room || GitQueue).wget(path, url, fileName)
            .error(reject)
            .then(resolve);

        return defer.promise;
    };

    this.move = function (src, path, room) {
        var defer = ZHPromise.defer();
        var reject = function (error) {
            defer.reject(error);
        };
        var resolve = function () {
            defer.resolve();
        };
        (room || GitQueue).move(src, path)
            .error(reject)
            .then(resolve);


        return defer.promise;
    };
}

Configuration(function (ContextRegister) {
    ContextRegister.register('WebSocketClient', WebSocketClient);
});



// FILE: assets/js/Service/Objects.js
function ObjectsService(requestDataSourceEngine, Utilities) {


    this.getObjectsBody = function(objectName, type, typeAbbreviation){
        var filter = [
            {
                "operator": "=",
                "name": "OBJECT_NAME",
                "value": objectName
            },
            {
                "operator": "=",
                "name": "OBJECT_TYPE",
                "value": type
            },
            {
                "operator": "=",
                "name": "TYPE_ABBREVIATION",
                "value": typeAbbreviation
            }
        ];
        return requestDataSourceEngine.getDataSource("/objectsBody" ,filter).then(function(response){
            return response.dataset.objectBody;
        });
    };

    this.loadTableObservations = function(tableName){
        var filter = [
            {
                "operator": "=",
                "name": "TABLE",
                "value": tableName
            }
        ];
        return requestDataSourceEngine.getDataSource("/tableObservations" ,filter).then(function(response){
            return response.dataset.findTableObservations[0];
        });
    };
    this.loadTableColumnObservations = function(columnName, tableName){
        var filter = [
            {
                "operator": "=",
                "name": "TABLE",
                "value": tableName
            },
            {
                "operator": "=",
                "name": "COLUMN",
                "value": columnName
            }
        ];
        return requestDataSourceEngine.getDataSource("/tableColumnObservations" ,filter).then(function(response){
            return response.dataset.findTableColumnObservations[0];
        });
    };

    this.saveTableObservation = function(widget){

        var dataset = [ {
            changed  : true,
            "row"    : widget.currentRow
        } ];

        return requestDataSourceEngine.setDataSource("/tableObservations" ,dataset).then(function(response){
            return response.dataset.tableObservations[0];
        });
    };

    this.saveTableColumnObservation = function(widget){

        var dataset = [ {
            changed  : true,
            "row"    : widget.currentRow
        } ];

        return requestDataSourceEngine.setDataSource("/tableColumnObservations" ,dataset).then(function(response){
            return response.dataset.tableColumnObservations[0];
        });
    };

    this.saveObjectsObservation = function(widget, type, objectName){

        var dataset = [ {
            changed  : true,
            "row"    : widget.currentRow,
            "type"   : type,
            "object_name" : objectName
        } ];

        return requestDataSourceEngine.setDataSource("/objectsObservation" ,dataset).then(function(response){
            return response.dataset.objectObservations;
        });
    };

}

Configuration(function(ContextRegister) {
    ContextRegister.register('ObjectsService', ObjectsService);
});

// FILE: assets/js/Service/Structure.js
function StructureService(WebSocketClient, requestDataSourceEngine, Utilities, ScreenService, MessageService, ZHPromise) {

    var self = this;
    var DEFAULT_FILE_NAME = 'DesignEstrutura';
    var executedScriptsCount, totalScripts;
    var errorScripts = [];
    var successScripts = [];
    var scriptTargets = [];
    var hasDefaultValue = true;
    var valitedScript = true;
    var uploadedScripts = [];
    var scriptArray = [];

    var validateCreateScriptExceptions = function(currentScript){
        var constraint = [];
        var value = [];
        var crt = currentScript.slice(currentScript.indexOf("CREATE"), currentScript.indexOf("(")).split(" ");
        if(currentScript.indexOf("CONSTRAINT") >= 0){
            var allConstraints = currentScript.split("CONSTRAINT");
            for(var j in allConstraints){
                if(j > 0){
                   constraint.push(currentScript.split("CONSTRAINT")[j].split(" ")[1]);
                }
            }
        }
        if(currentScript.indexOf("CREATE OR REPLACE") >= 0){
            value = [{
                "Name": crt[4],
                "Action": crt[0] + crt[1] + crt[2],
                "Origin": crt[3],
                "Constraint": constraint
            }];
        }else{
            var i = 0;
            if(currentScript.indexOf("UNIQUE") >= currentScript.indexOf("CREATE") && currentScript.indexOf("UNIQUE") <= currentScript.indexOf("INDEX") ){
                i = 1;
            }
            value = [{
                "Name": crt[2 + i],
                "Action": crt[0],
                "Origin": crt[1 + i],
                "Constraint": constraint
            }];
        }
        scriptTargets.push(value);
    };

    var validateAlterScriptExceptions = function(currentScript){
        var constraint = [];
        var value = [];
        if (currentScript.indexOf("ADD") > 0 && currentScript.indexOf("ADD") < 40) {
            var tableName = currentScript.substring(currentScript.indexOf("TABLE") + 5, currentScript.indexOf("ADD"));
            tableName = tableName.replace(/\s/g, '');
            var alterNames = currentScript.split("ADD");
            if (currentScript.indexOf("CONSTRAINT") >= 0) {
                var allConstraints = currentScript.split("CONSTRAINT");
                for (var i in allConstraints) {
                    if (i > 0) {
                        constraint.push(currentScript.split("CONSTRAINT")[i].split(" ")[1]);
                    }
                }
            }
            if(alterNames){
                for(var n = 1; n < alterNames.length; n++){
                    var alterName = alterNames[n].split(" ")[1];
                    alterName = alterName.replace(/\s/g, '');

                     value = [{
                        "Name": alterName,
                        "Action": "ADD",
                        "Origin": "ALTER TABLE",
                        "Table": tableName,
                        "Constraint": constraint
                    }];
                    scriptTargets.push(value);
                }
            }else {


                 value = [{
                    "Name": " ERROR LOADING FIELD",
                    "Action": "ADD",
                    "Origin": "ALTER TABLE",
                    "Table": tableName,
                    "Constraint": constraint
                }];
                scriptTargets.push(value);
            }

        }
    };

    var validateNotNullScriptExceptions = function(currentScript){
        if (currentScript.indexOf("ADD ") > 0){
            var notNulls = currentScript.split("NOT NULL");
            for(var d in notNulls) {
                if( notNulls[d].lastIndexOf("DEFAULT",  notNulls[d].length) < 0 && notNulls[d].lastIndexOf(" IS ",  notNulls[d].length) < 0 ){
                    if(d >= 0 && notNulls[d] !== ""){
                        hasDefaultValue = false;
                    }
                } else {
                    hasDefaultValue = true;
                    return 0;
                }
            }
        }
    };

    var validateScriptExceptions = function(currentScript, isUploaded){
        hasDefaultValue = true;
        currentScript = currentScript.toUpperCase();
        var currentScriptToValidate = currentScript.replace(/(\r\n|\n|\r|\\n)/gm, ' ');
        if(currentScript.indexOf("CREATE") >= 0){
            validateCreateScriptExceptions(currentScriptToValidate);
            if(isUploaded && (currentScript.indexOf("INDEX") < 0 || currentScript.indexOf("INDEX") > 40)){
                uploadedScripts.objectCriation = uploadedScripts.objectCriation ? (uploadedScripts.objectCriation + "\n / \n" + currentScript) : currentScript;
            }else if (isUploaded && currentScript.indexOf("INDEX") > 0){
                uploadedScripts.structureUpdate = uploadedScripts.structureUpdate ? (uploadedScripts.structureUpdate + "\n / \n" + currentScript) : currentScript;
            }
        }else {
            if(isUploaded && currentScript.indexOf("PACKAGE") > 0){
                uploadedScripts.packages = uploadedScripts.packages ? (uploadedScripts.packages + "\n / \n" + currentScript) : currentScript;
            }else if(isUploaded){
                uploadedScripts.structureUpdate = uploadedScripts.structureUpdate ? (uploadedScripts.structureUpdate + "\n / \n" + currentScript) : currentScript;
            }
        }
        if(currentScript.indexOf("ALTER TABLE") >= 0) {
            validateAlterScriptExceptions(currentScriptToValidate);
        }
        if(currentScript.indexOf("NOT NULL") > 0){
            validateNotNullScriptExceptions(currentScriptToValidate);
        }
    };

    var validateScriptBeforeParse = function(scripts, currentRow, logField, logInitMessage, isUploaded, scriptId){
        var messageCode = "LOG_CANCEL_PARSE";
        if(isUploaded){
          messageCode = "LOG_INIT_SCRIPT_UPLOAD";
        }

        return MessageService.getMessageByCode(messageCode).then(function(messageCode) {
            valitedScript = true;
            for (var i in scripts){
                validateScriptExceptions(scripts[i], isUploaded);
                var validateDuplicated = hasDuplicatedArray(scriptTargets);
                var scriptObj = {
                    SCRIPT : scripts[i]
                };

                if(isUploaded) {
                    currentRow.LOG =  messageCode;
                }

                ScreenService.restoreDefaultLoadingMessage();

                if((validateDuplicated[0] || hasDefaultValue === false) && !isUploaded) {
                    var logCustomMessage = ScreenService.i18n(logInitMessage, scriptObj);

                        if( i < 1){
                            if(!currentRow.LOG){
                                currentRow.LOG =  messageCode + '\n' + logCustomMessage;
                            }else {
                                currentRow.LOG +=  messageCode + '\n' + logCustomMessage;
                            }
                        } else if(i > 0){
                            currentRow.LOG += Utilities.LOG_COMMAND_SEPARATOR + '\n' + logCustomMessage;
                        } else {
                            currentRow.LOG += '\n' + logCustomMessage;
                        }

                    var scriptStatus = [
                        {
                            "SCRIPT" : scripts[i],
                            "STATUS": "E",
                            "ID" : scriptId
                        }
                    ];
                    scriptArray.push(scriptStatus);
                    valitedScript = false;
                    currentRow.IDRESULTSCRIPT = "PE";
                    currentRow.INVALID_SCRIPT = true;
                    var messageParam = "";
                    var execMessage = "";
                    if(validateDuplicated[0]){
                        execMessage = 'LOG_ERROR_SCRIPT_EXECUTION_REP_NAME';
                        var lastDuplicated = scriptTargets.pop()[0];
                        if(lastDuplicated.Constraint.length > 0){
                            messageParam = {
                                NAME : lastDuplicated.Constraint
                            };
                        } else{
                            messageParam = {
                                NAME : lastDuplicated.Name
                            };
                        }
                    } else if(hasDefaultValue === false){
                        execMessage = 'LOG_ERROR_SCRIPT_NOT_NULL';
                    }
                    incrementLog(currentRow, null, logField, execMessage, messageParam);
                }
            }
        });
    };

    var executeParseScript = function(scripts, totalScripts, currentRow, logField, index, defer, connType , scriptId){
        if(index == totalScripts) {
            if(errorScripts.length > 0) {
                 defer.resolve(Utilities.SCRIPT_EXECUTION_FAIL);
            } else {
                 defer.resolve(Utilities.SCRIPT_EXECUTION_SUCCESS);
            }
        } else {
            var currentScript = scripts[index].trim();
            if(Util.isEmptyOrBlank(currentScript)) {
                return executeParseScript(scripts, totalScripts, currentRow, logField, ++index, defer, connType , scriptId);
            } else{
                return MessageService.getMessageByCode('LOG_INIT_SCRIPT_EXECUTION').then(function(logInitMessage) {

                    if(connType == "PARSE" && index < 1){
                        var initLog = currentRow.LOG;
                        return validateScriptBeforeParse(scripts, currentRow, logField, logInitMessage, false, scriptId).then(function(){
                            if(valitedScript){
                                if(!currentRow.LOG){
                                    currentRow.LOG = initLog;
                                }
                               return  executeScripts(currentScript, currentRow, logField, connType, logInitMessage, scripts, totalScripts, index, defer, scriptId);

                            }
                        });
                    }else {
                        if(valitedScript){
                           return executeScripts(currentScript, currentRow, logField, connType, logInitMessage, scripts, totalScripts, index, defer, scriptId);
                        }
                    }

                });
            }
        }
    };

    var executeScripts = function(currentScript, currentRow, logField, connType, logInitMessage, scripts, totalScripts, index, defer, scriptId){

        return executeScript(currentScript, connType).then(function(scriptResponse){

            var scriptObj = {
                SCRIPT : currentScript
            };
            logInitMessage = ScreenService.i18n(logInitMessage, scriptObj);
            currentRow.LOG += '\n' + logInitMessage;

            var messageParam = null;
            var scriptResult = scriptResponse.dataset.scriptResult[0];
            var scriptResultId = scriptResponse.dataset.scriptResult.result;
            var execMessage = scriptResultId == "S" ? 'LOG_SUCCESS_SCRIPT_EXECUTION' : 'LOG_ERROR_SCRIPT_EXECUTION' ;

            ScreenService.restoreDefaultLoadingMessage();

            var scriptStatus = [
                {
                    "SCRIPT" : currentScript,
                    "STATUS": scriptResultId,
                    "ID" : scriptId
                }
            ];
            scriptArray.push(scriptStatus);
            incrementLog(currentRow, scriptResult, logField, execMessage, messageParam);
            if(scriptResultId == "E" && !errorScripts.includes(currentScript)) {
                if (!currentRow.SCRIPT_ERR) {
                    currentRow.SCRIPT_ERR = currentScript;
                } else {
                    currentRow.SCRIPT_ERR += '\n/\n' + currentScript;
                }
                errorScripts.push(currentScript);
            } else if(scriptResultId == "S" && !successScripts.includes(currentScript)){
                if(!currentRow.SCRIPT_SUC){
                    currentRow.SCRIPT_SUC = currentScript;
                } else {
                    currentRow.SCRIPT_SUC += '\n/\n' + currentScript;
                }
                successScripts.push(currentScript);
            }


            return executeParseScript(scripts, totalScripts, currentRow, logField, ++index, defer, connType, scriptId);
        }).catch(function (tableResponse) {
            var msg = ScreenService.i18n("Erro desconhecido");
            if(tableResponse && tableResponse.data && tableResponse.data.error) {
                msg = tableResponse.data.error;
            }
            incrementLog(currentRow,  msg, logField, "LOG_ERROR_SCRIPT_EXECUTION");
            errorScripts.push(currentScript);
            defer.resolve(Utilities.SCRIPT_EXECUTION_FAIL);
        });

    };
    var findAnyElementWithingArray = function(source, target){
        var result = source.filter(function(item){
            return target.indexOf(item) > -1;
        });
        return (result.length > 0);
    };

    var hasDuplicatedArray = function(a) {
        var k = [];
        for(var i in a){
            for(var j in a){
                if(a[i].length && a[j].length){
                    var matchedConstraint = findAnyElementWithingArray(a[i][0].Constraint, a[j][0].Constraint);
                    if(i!=j && a[i][0].Constraint !== "" && matchedConstraint){
                        if(k.indexOf(i) < 0){
                            k.push(i);
                        }
                    }
                }

                if(i!=j && JSON.stringify(a[i]) == JSON.stringify(a[j])){
                    if(k.indexOf(i) < 0){
                        k.push(i);
                    }
                }
            }
        }
        return k;
    };

    var buildErrorScriptForParse = function(currentRow, script, connType, logField, defer){
        return MessageService.getMessageByCode('LOG_INIT_SCRIPTS').then(function(messageInit) {
                var scripts      = removeCharsInScriptComments(script.SCRIPT);
                var scriptId = script.ID;
                scripts = getScriptsToExecute(scripts.split('\n'));
                executedScriptsCount = 0;
                totalScripts = scripts.length;
                messageInit = ScreenService.i18n(messageInit);
                if(!currentRow.LOG){
                    currentRow.LOG = messageInit;
                }
                return executeParseScript(scripts, scripts.length, currentRow, logField, 0, defer, connType, scriptId).then(function (response) {
                 return response;
                });
        });
    };

    this.executeScriptsFromError = function(currentRow, scriptsFromError, connType, logField){
        var defer      = ZHPromise.defer();
        successScripts = []; errorScripts = [];
        scriptTargets = [];
        scriptArray = [];
        valitedScript = true;
        currentRow.LOG = "";
        var functions = [];
        scriptsFromError.map(function(){
            functions.push(buildErrorScriptForParse);
        });
        return Utilities.doSyncMtFeatParametersForScript(functions, currentRow, scriptsFromError, connType, logField, defer).then(function(){
            return scriptArray;
        });

    };
    this.executeScript = function(currentRow, scripts, connType, logField) {
        var defer      = ZHPromise.defer();
        successScripts = []; errorScripts = [];
        scriptTargets = [];
        valitedScript = true;
        scripts       = removeCharsInScriptComments(scripts);
        scripts = getScriptsToExecute(scripts.split('\n'));
        scriptArray = [];
        executedScriptsCount = 0;
        totalScripts = scripts.length;
        return MessageService.getMessageByCode('LOG_INIT_SCRIPTS').then(function(messageInit) {
            messageInit = ScreenService.i18n(messageInit);
            currentRow.LOG = messageInit;
            currentRow.SCRIPT_ERR = "";
            currentRow.SCRIPT_SUC = "";
            return executeParseScript(scripts, scripts.length, currentRow, logField, 0, defer, connType, null).then(function (response) {
                ScreenService.restoreDefaultLoadingMessage();
                return scriptArray;
            });
        });
    };

    var getScriptsToExecute = function(tempScripts) {
        var scripts = [];
        var idx = 0;
        tempScripts.forEach(function(tempScript) {
            var trimTempScript = tempScript.trim();
            if(trimTempScript !== '') {
                if (trimTempScript[0] == '/' && trimTempScript[1] != '*') {
                    idx++;
                } else {
                    scripts[idx] = scripts[idx] ? scripts[idx] + '\n' + tempScript : tempScript;
                }
            }
        });
        //reset keys
        return scripts.filter(function() { return true; });
    };

    var removeCharsInScriptComments = function (currentScript) {
        var idx         = 0;
        var script      = '';
        var tempScript  = '';

        while(idx < currentScript.length) {
            if(currentScript[idx] == "'") {
                tempScript = getTempScript(currentScript, idx + 1, "'");
                script += "'" + tempScript.SCRIPT + "'";
                idx = idx + tempScript.LAST_IDX + 2;
            } else if(currentScript[idx] == "-" && currentScript[idx + 1] == "-") {
                tempScript = getTempScript(currentScript, idx + 2, "\n");
                script += "--" + replaceQuotationMarks(tempScript.SCRIPT) + "\n";
                idx = idx + tempScript.LAST_IDX + 3;
            } else if(currentScript[idx] == "/" && currentScript[idx + 1] == "*") {
                tempScript = getTempScript(currentScript, idx + 2, "*/");
                script += "/*" + replaceQuotationMarks(tempScript.SCRIPT) + "*/";
                idx = idx + tempScript.LAST_IDX + 4;
            } else {
                script += currentScript[idx];
                idx++;
            }
        }
        return script;
    };
    var getIndecesOf = function(searchStr, str){
        var searchStrLen = searchStr.length;
        if (searchStrLen === 0) {
            return [];
        }
        var startIndex = 0, index, indices = [];
        while ((index = str.indexOf(searchStr, startIndex)) > -1) {
            indices.push(index);
            startIndex = index + searchStrLen;
        }
        return indices;
    };

    this.validateScriptsComments = function (currentScript) {

        //True -> validação valida, false -> validação Invalida
        if(currentScript.match(/\/\*/g) && currentScript.match(/\*\//g)){
            var opencomment = currentScript.match(/\/\*/g).length;
            var closecomment = currentScript.match(/\*\//g).length;
            if(opencomment == closecomment){
                var openCommentIndices = getIndecesOf("/*", currentScript);
                var closeCommentIndices = getIndecesOf("*/", currentScript);
                var isValid =  true;
                for(i=0; i <= (openCommentIndices.length);i++){
                    if(closeCommentIndices[i] < openCommentIndices[i] || closeCommentIndices[i] > openCommentIndices[i+1] ){
                        isValid = false;
                    }
                }
                return isValid;
            }else{
                return false;
            }
        }else {
            return !(currentScript.match(/\*\//g) || currentScript.match(/\/\*/g));
        }

    };


    var getTempScript = function(currentScript, idx, endDelimiter) {
        var endIndexOff = currentScript.substr(idx).indexOf(endDelimiter);
        return {
            'SCRIPT'  : currentScript.substr(idx).substr(0, endIndexOff),
            'LAST_IDX': endIndexOff
        };
    };

    var replaceQuotationMarks = function(script) {
        return script.replace(/'/g, '').replace(/"/g, '');
    };
    var getLogfiles = function(row, type, oldLog, repository) {
        if(!row.LOG && !oldLog) {
            return ZHPromise.when({
                "value" : false
            });
        }
        return getServerLogFileContents(row, row.OPERATT, repository, row.LOG, type).then(function(log){
            var fullLog = "";
            if(oldLog){
                fullLog = oldLog +  log;
            } else {
                fullLog =  log;

            }
            var arr = [];
            arr.push(buildLogDocument(row,fullLog));
            return arr;
        });

    };

    this.downloadScriptLog = function(id, description, logDir) {
        description = description ? description.replace(/#/g, '').replace(/&/g, '').trim() : "";
        var routeStr           = '/downloadScriptLog';
        var requestTypeStr     = '?requestType=DataSet';
        var datasetFileNameStr = '&dataset[0][FILE_NAME]=' + getLogFileName(id, description);
        var datasetFileDirStr  = '&dataset[0][FILE_DIR]=' + logDir;
        window.open(window.serviceUrl + routeStr + requestTypeStr + datasetFileNameStr + datasetFileDirStr);
    };

    this.downloadScript = function(id) {
        var routeStr       = '/downloadScript';
        var requestTypeStr = '?requestType=DataSet';
        var datasetIdStr   = '&dataset[0][ID]=' + id;
        window.open(window.serviceUrl + routeStr + requestTypeStr + datasetIdStr);
    };

    var getLogFileName = function(id, description) {
        return (description || DEFAULT_FILE_NAME) + '_' + id + '.log';
    };

    this.getFullLog = function(dirLog){
        var filter = [
            {
                "operator": "=",
                "name": "DIRLOG",
                "value": dirLog
            }
        ];
        return requestDataSourceEngine.getDataSource('/getFullLog', filter).then(function(response){
            return response.dataset.logFileContents[0];
        });
    };
    var getLogHeader = function(repository){
        var filter = [
            {
                "name" : "ID",
                "operator": "=",
                "value": repository
            }
        ];
        return requestDataSourceEngine.getDataSource("/getUserAndRepository" ,filter).then(function(response){
            return response.dataset.logHeader;
        });
    };

    var getServerLogFileContents = function(row, user, repository, currentLogContents, typeLog) {
        return getLogHeader(repository).then(function(logHeader){
            var log = currentLogContents ? currentLogContents : "";
            var logFileContents = '\n-----> Design Estrutura: ' + row.ID + '-' + row.DSDESESTRUTURA;
            logFileContents += ' | ' + ScreenService.i18n('Usuário') + ': ' + logHeader[1];
            logFileContents += ' | ' + ScreenService.i18n('Repositório') + ': ' + logHeader[0];
            logFileContents += ' | ' + ScreenService.i18n('Data') + ': ' + Utilities.getCurrentDateWithTime();
            logFileContents += ' | ' + typeLog + ' <-----\n\n' + log + '\n\n';
            return logFileContents;
        });
    };

    var buildLogDocument = function(row,rowLog){
        var log = Utilities.transliterateString(rowLog).replace(/[\u00A0-\u2666]/g, '');
        return  {
            name              : (row.DSDESESTRUTURA || DEFAULT_FILE_NAME) + '.log',
            b64File           : 'data:text/plain;base64,' + Utilities.b64Encode(log),
            lastModified      : Utilities.getCurrentDateInMilliseconds(),
            lastModifiedDate  : Utilities.getCurrentFormattedDateForAttachment(),
            size              : Utilities.byteLength(rowLog),
            type              : 'text/plain',
            webkitRelativePath: ''
        };
    };

    this.uploadScripts = function(scripts, logField, currentRow, repository){
        var defer = ZHPromise.defer();
        var resolve = function(success) {
            defer.resolve(success);
        };
        var reject = function(error) {
            defer.reject(error);
        };

        scripts         = removeCharsInScriptComments(scripts);
        var scriptarray = getScriptsToExecute(scripts.split('\n'));
        executedScriptsCount = 0;
        totalScripts = scriptarray.length;
        uploadedScripts = [];

        divideScriptsUpload(scriptarray, logField, currentRow, repository).then(function() {
            var script = [] ;
            currentRow.SCRIPT_SUC = "";
            currentRow.DSDESESTRUTURA = currentRow.DOCUMENT[0].name;
            currentRow.SCRIPT_ERR = "";
            currentRow.IDATIVO = "S";
            currentRow.IDRESULTSCRIPT = "PD";
            currentRow.REPOSITORIO_ID = repository;

            script.push(uploadedScripts.packages);
            script.push(uploadedScripts.structureUpdate);
            if(uploadedScripts.objectCriation){
                uploadedScripts.objectCriation = getScriptsToExecute(uploadedScripts.objectCriation.split('\n'));
                uploadedScripts.objectCriation.forEach(function (obj) {
                    script.push(obj);
                });
            }

            currentRow.SCRIPT = script;

            saveUploadedScript(currentRow, repository).then( function() {
                resolve();
            }).catch(reject);
        }).catch(reject);

        return defer.promise;
    };

    var saveUploadedScript = function(currentRow, repository){
        currentRow.REPOSITORIO_ID = repository;

        var dataset = [ {
            changed : true,
            "row"   : currentRow
        } ];
        return requestDataSourceEngine.setDataSource('/structureUpload', dataset).then(function(response) {
            return response.dataset.savedScripts[0];
        });

    };

    var divideScriptsUpload = function (scripts, logField, currentRow){
        return MessageService.getMessageByCode('LOG_INIT_SCRIPT_EXECUTION').then(function(logInitMessage) {
            return validateScriptBeforeParse(scripts, currentRow , logField, logInitMessage, true);
        });
    };

    var buildScriptSql = function(nrscript,rowScript){
        var script = Utilities.transliterateString(rowScript).replace(/[\u00A0-\u2666]/g, '');
        return  {
            name              : nrscript + '.sql',
            b64File           : 'data:text/plain;base64,' + Utilities.b64Encode(script),
            lastModified      : Utilities.getCurrentDateInMilliseconds(),
            lastModifiedDate  : Utilities.getCurrentFormattedDateForAttachment(),
            size              : Utilities.byteLength(rowScript),
            type              : 'text/plain',
            webkitRelativePath: ''
        };
    };

    this.saveScript = function(currentRow, type, oldLog, repository, scriptArray) {

        currentRow.REPOSITORIO_ID = repository;
        return getLogfiles(currentRow, type, oldLog, repository).then(function(logFiles) {
            var dataset = [ {
                changed : true,
                "row"   : currentRow,
                "LOG"   : logFiles,
                "SCRIPTARRAY" : scriptArray
            } ];
            return requestDataSourceEngine.setDataSource('/structure', dataset).then(function(response) {
                return response.dataset.savedScripts[0];
            });
        });
    };

    this.updateScript = function(currentRow, scriptArray, type, oldLog, repository) {

        currentRow.REPOSITORIO_ID = repository;
        return getLogfiles(currentRow, type, oldLog, repository).then(function(logFiles) {
            var dataset = [ {
                changed : true,
                "row"   : currentRow,
                "LOG"   : logFiles,
                "SCRIPTARRAY" : scriptArray
            } ];
            return requestDataSourceEngine.setDataSource('/structureUpdate', dataset).then(function(response) {
                return response.dataset.savedScripts[0];
            });
        });
    };

    this.getErrorsScripts = function(currentRow) {
            var dataset = [ {
                "operator": "=",
                "name": "ID",
                "value": currentRow.ID
            } ];
            return requestDataSourceEngine.getDataSource('/getScriptsWithError', dataset).then(function(response) {
                return response.dataset.Scripts;
            });
    };

    var executeScript = function(script, connType) {
        var dataset = [ {
            changed : true,
            "SCRIPT": script,
            "connectionType": connType

        } ];
        return requestDataSourceEngine.setDataSource('/structureScript', dataset);
    };

    var incrementLog = function (structureUpdateRow, outputScript, logField, messageCode, messageParam) {
        var commandSeparator = "";
        if(messageCode == "LOG_SUCCESS_SCRIPT_EXECUTION" || messageCode == "LOG_ERROR_SCRIPT_EXECUTION") {
            commandSeparator = Utilities.LOG_COMMAND_SEPARATOR;
        }

        return MessageService.getMessageByCode(messageCode).then(function(messageCode){
            if(messageParam){
                messageCode = ScreenService.i18n(messageCode, messageParam);
            } else {
                messageCode = ScreenService.i18n(messageCode);
            }

            if(outputScript) {
                structureUpdateRow.LOG += outputScript + '\n\n';
            }
            structureUpdateRow.LOG += messageCode + commandSeparator;

            var loadMessage = ScreenService.i18n("Executando Scripts");
            return ScreenService.changeLoadingMessage(loadMessage + ": " + (++executedScriptsCount) + '/' + totalScripts);
        });

    };

    this.validateWebSocketDirectory = function(idRep){

        var filter = [
            {
                "operator": "=",
                "name": "IDREP",
                "value": idRep
            }
        ];
        return requestDataSourceEngine.getDataSource('/validateWebSocketDirectory', filter).then(function(response){
            return response.dataset.validateWebSocketDirectory[0];
        });
    };


    this.removeTempFile = function(url) {
        var filter = [ {
            "operator": "=",
            "name": "Dir",
            "value": url
        } ];
        requestDataSourceEngine.getDataSource('/removeScriptOnTempFile', filter);
    };

    this.downloadByWebSocket = function(clientEmail, dir, structure) {
        var defer = ZHPromise.defer();
        var resolve = function(success) {
            defer.resolve(success);
        };
        var reject = function(error) {
            defer.reject(error);
        };

        var dataset = [ {
            changed  : true,
            "row"    : structure,
            "SCRIPT" : [buildScriptSql(structure.NRSCRIPT, structure.SCRIPT)]
        } ];

        requestDataSourceEngine.setDataSource('/scriptOnTempFile', dataset).then(function(response) {
            var url = response.dataset.scriptOnTempFile[0];
            var pathFileServer = response.dataset.scriptOnTempFile[1];
            var directorySeparator = "";

            var room = "";
            var fileName = structure.NRSCRIPT + ".sql";
            if(dir.includes("/")){
                directorySeparator = "/";
            }else {
                directorySeparator = "\\";
            }
            var src  = dir + directorySeparator + fileName + ".tar.gz";
            var dest = dir + directorySeparator + fileName;

            WebSocketClient.loginInServer(clientEmail).then(function (rooms) {
                room = rooms;
                return WebSocketClient.wget(dir, url, fileName, room);
            }).then(function () {
                return WebSocketClient.move(src, dest, room);
            }).then(function() {
                resolve(pathFileServer);
            }).catch(reject);
        }).catch(reject);

        return defer.promise;
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('StructureService', StructureService);
});

// FILE: assets/js/Controller/Functions.js
function FunctionsController(ScreenService, ObjectsService, localStorageEngine,  templateManager, Utilities) {

    // ------------------------ GET WIDGETS ------------------------ \\
    var getFunctionsWidget = function () {
        return templateManager.container.getWidget('FUNCTIONS');
    };
    // --------------------------- END -------------------------------//

    this.functionsWidgetOnEnter = function() {
        ScreenService.showLoader();
        localStorageEngine.getLocalVar('loginPromise').then(function () {
            ScreenService.hideLoader();
            var widget = getFunctionsWidget();
            Utilities.applyDefaultFilterOnEnter(widget, true);
        });
    };

    this.functionsBasicBackAction = function(){
        ScreenService.closeSwipe();
    };

    this.functionBasicEditAction = function(){
        var widget = getFunctionsWidget();
        localStorageEngine.setLocalVar('functionObservation', widget.currentRow.OBSERVATION);
        widget.edit();
    };

    this.functionBasicCancelAction = function(){
        getFunctionsWidget().currentRow.OBSERVATION = localStorageEngine.getLocalVar('functionObservation');
        getFunctionsWidget().view();
    };

    this.functionsWidgetFilterAfterApply = function (){
        Utilities.setFilterInit(getFunctionsWidget());
    };

    this.functionsWidgetAfterMoveRow = function (){
        var widget = getFunctionsWidget();
        var name = widget.currentRow.OBJECT_NAME;
        var type = 'FUNCTION';
        ObjectsService.getObjectsBody(name, type, 'F').then(function(objectBody){
            widget.currentRow.OBJECT_BODY = objectBody.BODY;
            if(objectBody.OBSERVATION){
                widget.currentRow.OBSERVATION = objectBody.OBSERVATION;
            }
        });
    };

    this.functionActionSaveObservation = function (){
        var widget = getFunctionsWidget();
        ObjectsService.saveObjectsObservation(widget, 'F', widget.currentRow.OBJECT_NAME).then(function(response){
            widget.view();
        }).catch(Utilities.showMessageErrorOfBackEnd);

    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('FunctionsController', FunctionsController);
});

// FILE: assets/js/Controller/Index.js
function IndexController(ScreenService, localStorageEngine,  templateManager, Utilities, ObjectsService) {

    // ------------------------ GET WIDGETS ------------------------ \\
    var getIndexWidget = function () {
        return templateManager.container.getWidget('INDEX');
    };
    var getIndexColumnsWidget = function () {
        return templateManager.container.getWidget('INDEX_COLUMNS');
    };
    // --------------------------- END -------------------------------//

    this.indexWidgetOnEnter = function() {
        ScreenService.showLoader();
        localStorageEngine.getLocalVar('loginPromise').then(function () {
            ScreenService.hideLoader();
            var widget = getIndexWidget();
            Utilities.applyDefaultFilterOnEnter(widget, true);
        });
    };

    this.indexActionSaveObservation = function (){
        var widget = getIndexWidget();
        ObjectsService.saveObjectsObservation(widget, 'I', widget.currentRow.INDEX_NAME).then(function(response){
            widget.view();
        }).catch(Utilities.showMessageErrorOfBackEnd);

    };

    this.indexWidgetAfterMoveRow = function (){
        var widget = getIndexWidget();
        var name = widget.currentRow.INDEX_NAME;
        ObjectsService.getObjectsBody(name, null, 'I').then(function(objectBody){
            widget.currentRow.BODY = objectBody.BODY;
            widget.currentRow.OBSERVATION = objectBody.OBSERVATION;

        });
    };

    this.columnWidgetAfterMoveRow = function (){
        var widget = getIndexColumnsWidget();
        ObjectsService.loadTableColumnObservations(widget.currentRow.INDEX_NAME, widget.currentRow.TABLE_NAME).then(function(response){
            widget.currentRow.OBSERVATION = response;
        });
    };

    this.indexBasicBackAction = function(){
        ScreenService.closeSwipe();
    };

    this.indexBasicEditAction = function(){
        var widget = getIndexWidget();
        localStorageEngine.setLocalVar('indexObservation', widget.currentRow.OBSERVATION);
        widget.edit();
    };

    this.indexBasicCancelAction = function(){
        getIndexWidget().currentRow.OBSERVATION = localStorageEngine.getLocalVar('indexObservation');
        getIndexWidget().view();
    };

    this.indexColumnWidgetOnEnter = function(){
        getIndexColumnsWidget().reload();
    };

    this.indexWidgetFilterAfterApply = function (){
        Utilities.setFilterInit(getIndexWidget());
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('IndexController', IndexController);
});

// FILE: assets/js/Controller/Job.js
function JobController(ScreenService, ObjectsService, localStorageEngine,  templateManager, Utilities) {

    // ------------------------ GET WIDGETS ------------------------ \\
    var getJobWidget = function () {
        return templateManager.container.getWidget('JOB');
    };
    // --------------------------- END -------------------------------//

    this.jobWidgetOnEnter = function() {
        ScreenService.showLoader();
        localStorageEngine.getLocalVar('loginPromise').then(function () {
            ScreenService.hideLoader();
            var widget = getJobWidget();
            Utilities.applyDefaultFilterOnEnter(widget, true);
        });
    };

    this.jobBasicBackAction = function(){
        ScreenService.closeSwipe();
    };

    this.jobWidgetFilterAfterApply = function (){
        Utilities.setFilterInit(getJobWidget());
    };

    this.jobBasicCancelAction = function (){
        getJobWidget().currentRow.OBSERVATION = localStorageEngine.getLocalVar('jobObservation');
        getJobWidget().view();
    };

    this.jobBasicEditAction = function (){
        var widget = getJobWidget();
        localStorageEngine.setLocalVar('jobObservation', widget.currentRow.OBSERVATION);
        widget.edit();
    };

    this.jobWidgetAfterMoveRow = function (){
        var widget = getJobWidget();
        var name = widget.currentRow.JOB_NAME;
        var type = 'PROCOBJ';
        ObjectsService.getObjectsBody(name, type, 'J').then(function(objectBody){
            widget.currentRow.BODY= objectBody.BODY;
            if(objectBody.OBSERVATION){
                widget.currentRow.OBSERVATION = objectBody.OBSERVATION;
            }
        });
    };

    this.jobActionSaveObservation = function (){
        var widget = getJobWidget();
        ObjectsService.saveObjectsObservation(widget, 'J', widget.currentRow.JOB_NAME).then(function(response){
            widget.view();
        }).catch(Utilities.showMessageErrorOfBackEnd);

    };

}

Configuration(function(ContextRegister) {
    ContextRegister.register('JobController', JobController);
});

// FILE: assets/js/Controller/Procedure.js
function ProcedureController(ScreenService, ObjectsService, localStorageEngine,  templateManager, Utilities) {

    // ------------------------ GET WIDGETS ------------------------ \\
    var getProcedureWidget = function () {
        return templateManager.container.getWidget('PROCEDURE');
    };
    // --------------------------- END -------------------------------//

    this.procedureWidgetOnEnter = function() {
        ScreenService.showLoader();
        localStorageEngine.getLocalVar('loginPromise').then(function () {
            ScreenService.hideLoader();
            var widget = getProcedureWidget();
            //ZhFilterPreferences.setOriginalFilter(widget.id, widget.currentRow);
            Utilities.applyDefaultFilterOnEnter(widget, true);
        });
    };

    this.procedureBasicBackAction = function(){
        ScreenService.closeSwipe();
    };

    this.procedureBasicEditAction = function(){
        var widget = getProcedureWidget();
        localStorageEngine.setLocalVar('procedureObservation', widget.currentRow.OBSERVATION);
        widget.edit();
    };

    this.procedureBasicCancelAction = function(){
        getProcedureWidget().currentRow.OBSERVATION = localStorageEngine.getLocalVar('procedureObservation');
        getProcedureWidget().view();
    };

    this.procedureWidgetFilterAfterApply = function (){
        Utilities.setFilterInit(getProcedureWidget());
    };

    this.procedureWidgetAfterMoveRow = function (){
        var widget = getProcedureWidget();
        var name = widget.currentRow.OBJECT_NAME;
        var type = 'PROCEDURE';
        ObjectsService.getObjectsBody(name, type, 'P').then(function(objectBody){
            widget.currentRow.OBJECT_BODY = objectBody.BODY;
            if(objectBody.OBSERVATION){
                widget.currentRow.OBSERVATION = objectBody.OBSERVATION;
            }
        });
    };

    this.procedureActionSaveObservation = function (){
        var widget = getProcedureWidget();
        ObjectsService.saveObjectsObservation(widget, 'P', widget.currentRow.OBJECT_NAME).then(function(response){
            widget.view();
        }).catch(Utilities.showMessageErrorOfBackEnd);

    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('ProcedureController', ProcedureController);
});

// FILE: assets/js/Controller/Repositorio.js
function RepositorioController(ScreenService, localStorageEngine, MessageService, templateManager, Utilities) {

    // ------------------------ GET WIDGETS ------------------------ \\
    var getRepositorioWidget = function () {
        return templateManager.container.getWidget('REPOSITORIO');
    };
    // --------------------------- END -------------------------------//

    this.repositorioWidgetOnEnter = function() {
        ScreenService.showLoader();
        localStorageEngine.getLocalVar('loginPromise').then(function () {
            ScreenService.hideLoader();
            var widget = getRepositorioWidget();
            Utilities.applyDefaultFilterOnEnter(widget, true);
        });
    };

    this.repositorioActionEventAdd = function() {
        getRepositorioWidget().newRow();
    };

    this.repositorioBasicBackAction = function(){
        ScreenService.closeSwipe();
    };

    this.repositorioBasicCancelAction = function(){
        var widget = getRepositorioWidget();
        if(widget.currentRow.__is_new){
            ScreenService.closeSwipe();
        } else {
            widget.view();
        }
        getRepositorioWidget().reload();
    };

    this.repositorioWidgetFilterAfterApply = function (){
        Utilities.setFilterInit(getRepositorioWidget());
    };

    this.repositorioActionEventEdit = function (){
        var widget = getRepositorioWidget();
        widget.edit();
    };

    this.repositorioActionEventDelete = function () {
        MessageService.confirmMessageByCode('ALE_CONFIRM_REMOVE_REGISTER', null,
            {
                type: 'QUESTION',
                success: function() {
                    Utilities.genericDeleteEvent(getRepositorioWidget()).then(function() {
                        MessageService.successNotificationByCode('SUC_DELETED');
                    }).catch(Utilities.showMessageErrorOfBackEnd);
                },
                fail: function () {}
            });
    };

    this.repositorioActionEventSave = function (){
        var widget = getRepositorioWidget();
        widget.saveRow().then(function(response){
            if(widget.currentRow.__is_new){
                ScreenService.closeSwipe();
            }else {
                widget.view();
            }
            widget.reload();
            MessageService.successNotificationByCode('SUC_CREATED');
        }).catch(Utilities.showMessageErrorOfBackEnd);
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('RepositorioController', RepositorioController);
});

// FILE: assets/js/Controller/Sequence.js
function SequenceController(ScreenService, localStorageEngine,  templateManager, Utilities, ObjectsService) {

    // ------------------------ GET WIDGETS ------------------------ \\
    var getSequenceWidget = function () {
        return templateManager.container.getWidget('SEQUENCE');
    };
    // --------------------------- END -------------------------------//

    this.sequenceWidgetOnEnter = function() {
        ScreenService.showLoader();
        localStorageEngine.getLocalVar('loginPromise').then(function () {
            ScreenService.hideLoader();
            var widget = getSequenceWidget();
            Utilities.applyDefaultFilterOnEnter(widget, true);
        });
    };

    this.sequenceBasicBackAction = function(){
        ScreenService.closeSwipe();
    };

    this.sequenceBasicEditAction = function(){
        var widget = getSequenceWidget();
        localStorageEngine.setLocalVar('sequenceObservation', widget.currentRow.OBSERVATION);
        widget.edit();
    };

    this.sequenceBasicCancelAction = function(){
        getSequenceWidget().currentRow.OBSERVATION = localStorageEngine.getLocalVar('sequenceObservation');
        getSequenceWidget().view();
    };

    this.sequenceWidgetFilterAfterApply = function (){
        Utilities.setFilterInit(getSequenceWidget());
    };

    this.sequenceWidgetAfterMoveRow = function (){
        var widget = getSequenceWidget();
        var name = widget.currentRow.SEQUENCE_NAME;
        ObjectsService.getObjectsBody(name, null, 'S').then(function(objectBody){
            if(objectBody.OBSERVATION){
                widget.currentRow.OBSERVATION = objectBody.OBSERVATION;
            }
        });
    };

    this.sequenceActionSaveObservation = function (){
        var widget = getSequenceWidget();
        ObjectsService.saveObjectsObservation(widget, 'S', widget.currentRow.SEQUENCE_NAME).then(function(response){
            widget.view();
        }).catch(Utilities.showMessageErrorOfBackEnd);

    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('SequenceController', SequenceController);
});

// FILE: assets/js/Controller/Structure.js
function StructureController(ZHPromise, WebSocketClient, ScreenService, StructureService, localStorageEngine, MessageService, templateManager, Utilities, ZhFilterPreferences, metaDataFactory, requestDataSourceEngine) {

    var self = this;
    var logValue = "";
    var tempScript = null;
    var isFromErr = null;
    var logErrEdit = false;
    var logExec = false;
    var scriptsFieldsNames = [];

    // ------------------------ GET WIDGETS ------------------------ \\

    var getStructureWidget = function () {
        return templateManager.container.getWidget('ESTRUTURA');
    };
    var getStripeWidget = function () {
        return templateManager.container.getWidget('STRIPE_NAVIGATOR');
    };
    var getRepositoryWidget = function () {
        return templateManager.container.getWidget('REPOSITORY');
    };
    var getViewLogWidget = function () {
        return templateManager.container.getWidget('VIEWLOG');
    };

    var getWidgetScriptUpload = function() {
        return templateManager.container.getWidget('SCRIPT_UPLOAD');
    };

    var getWidgetScriptFix = function() {
        return templateManager.container.getWidget('SCRIPT_FIX');
    };
    // -------------------------- Fields --------------------------- \\

    var getScriptFromStructureWidget = function() {
        return getStructureWidget().getField('SCRIPT');
    };
    var getDescriptionFromStructureWidget = function() {
        return getStructureWidget().getField('DSDESESTRUTURA');
    };
    var getIdAtivoFromStructureWidget = function() {
        return getStructureWidget().getField('IDATIVO');
    };
    var getLogFromStructureWidget = function() {
        return getStructureWidget().getField('LOG');
    };
    var getLogFromLogWidget = function() {
        return getViewLogWidget().getField('LOG');
    };
    var getRepositoryIdFromRepositoryWidget = function() {
        return getRepositoryWidget().getField('REPOSITORIO_ID');
    };
    var getRepositoryIdFromEstruturaWidget = function() {
        return getStructureWidget().getField('REPOSITORIO_ID');
    };

    // --------------------------- ACTIONS ------------------------- \\

    var getActionParsefromStructureWidget = function () {
        return getStructureWidget().getAction('EXECSCRIPT');
    };
    var getActionBaixarLogfromStructureWidget = function () {
        return getStructureWidget().getAction('LOGDOWNLOAD');
    };
    var getActionBaixarScriptfromStructureWidget = function () {
        return getStructureWidget().getAction('SCRIPTDOWNLOAD');
    };
    var getActionErrEditStructureWidget = function () {
        return getStructureWidget().getAction('ERREDIT');
    };
    var getActionViewLogStructureWidget = function () {
        return getStructureWidget().getAction('VIEWLOG');
    };
    var getActionExecuteStructureWidget = function () {
        return getStructureWidget().getAction('EXECUTE');
    };
    var getActionEditStructureWidget = function () {
        return getStructureWidget().getAction('EDIT');
    };
    var getActionSaveDescStructureWidget = function () {
        return getStructureWidget().getAction('SAVEDESC');
    };
    var getActionErrEditViewLogWIdget = function () {
        return getViewLogWidget().getAction('ERREDIT');
    };
    var getActionExecViewLogWIdget = function () {
        return getViewLogWidget().getAction('EXEC');
    };
    
    var getActionRemoveStructureWidget = function () {
        return getStructureWidget().getAction('REMOVE');
    };
    var getFieldIssuesStructureWidget = function(){
        return getStructureWidget().getField('ISSUES');
    };
    var getFieldIssuesUplodStructureWidget = function(){
        return getWidgetScriptUpload().getField('ISSUES_UPLOAD');
    };

    // --------------------------- END ----------------------------- \\


    this.getScriptFromErrorField = function (projectId, widget) {
        return metaDataFactory.factoryField(this.getScriptFromError(projectId), widget);
    };
    // ---------------------- Dynamic Field ------------------- //
    
    this.getScriptFromError = function (scriptId) {
        return {
            "name": "SCRIPT_" + scriptId,
            "label": "Script",
            "readOnly": false,
            "template": "field/textarea.html",
            "help": "Atalhos :<br> <br> Ctrl-F: Pesquisar <br> Ctrl-Space: Autocomplete <br> Alt-G: Pesquisar Linha <br> F11: Tela Cheia",
            "maxWidth": "15%",
            "textRows": 10,
            "mtIsGeneration": true,
            "disableOrdering": true,
            "isVisible": true,
            "showOnList": true,
            "showOnView": true,
            "class": 12,
            "filterProperties": {
                "help": false,
                "class": 5,
                "sequenceInForm": 3,
                "isVisible": true,
                "template": "field/text-edit.html"
            }
        };
    };

    // -=======================================================-//

    this.structureCloseSwipe = function() {
        ScreenService.closePopup();
    };

    this.structureCloseFixErrorsPopup = function() {
        var widgetFix = getWidgetScriptFix();
        var fields = widgetFix.fields;
        widgetFix.fields = self.mtGenerationProjectIdFilter(fields);
        ScreenService.closePopup(0);
        getStructureWidget().activate();
    };

    this.structureActionChangeRep = function() {
        var repositoryWidget = getRepositoryWidget();
        ScreenService.openPopup(repositoryWidget);
    };

    this.structureWidgetOnEnter = function() {
        ScreenService.showLoader();
        localStorageEngine.getLocalVar('loginPromise').then(function () {
            var nrRepository = localStorage.getItem('repositorio');
            ScreenService.hideLoader();
            var widget = getStructureWidget();
            widget.currentRow = {
                "IDRESULTSCRIPT": ['PD' , 'PS', 'PE', 'EE'],
                "IDATIVO": 'S'
            };
            ZhFilterPreferences.setOriginalFilter(widget.id, widget.currentRow);
            Utilities.applyDefaultFilterOnEnter(widget, true);
        });
    };

    this.structureWidgetOnEnterOnce = function() {
        var repositoryWidget = getRepositoryWidget();
        var repository = localStorage.getItem('repositorio');
        if(!repository) {
            ScreenService.openPopup(repositoryWidget);
        } else {
            loadStripeRepositorio();
        }
    };

    var loadStripeRepositorio = function(){
        var stripeWidget = getStripeWidget();
        var repository = localStorage.getItem('repositorio');
        localStorageEngine.setLocalVar('repositorio', repository);
        stripeWidget.reload().then(function(){
            stripeWidget.moveToFirst();
        });
    };

    this.structureActionSaveRepository = function() {
        var stripeWidget = getStripeWidget();
        var repositoryWidget = getRepositoryWidget();
        var widgetStructure = getStructureWidget();

        if(repositoryWidget.currentRow.REPOSITORIO_ID) {
            localStorage.setItem('repositorio', repositoryWidget.currentRow.REPOSITORIO_ID);
            var repository = localStorage.getItem('repositorio');
            localStorageEngine.setLocalVar('repositorio', repository);
        }
        loadStripeRepositorio();
        ScreenService.closePopup(0);
        widgetStructure.reload();

    };

    this.structureBasicBackAction = function(){
        var widget = getStructureWidget();
        var scriptField = getScriptFromStructureWidget();
        scriptField.readOnly = false;
        ScreenService.closeSwipe();
        validateActions();
        widget.list();
    };

    this.structureViewLogBackAction = function(){
        logErrEdit = false;
        logExec = false;
        ScreenService.closePopup(0);
        validateActions();
    };

    this.structureBasicCancelAction = function(){
        localStorageEngine.setLocalVar('CodeMirror', null);
        var widget = getStructureWidget();
        if(tempScript) {
            widget.currentRow.SCRIPT = tempScript;
            tempScript = null;
        }
        if(widget.currentRow.__is_new) {
            ScreenService.closeSwipe();
            widget.reload();
        } else {
            widget.view();
        }

        isFromErr = false;
        validateActions();
    };

    this.structureWidgetFilterAfterApply = function (){
        Utilities.setFilterInit(getStructureWidget());
    };

    this.structureActionEventAdd = function () {
        var repositorioId = localStorage.getItem('repositorio');
        if(!repositorioId) {
            ScreenService.openPopup(getRepositoryWidget());
        } else {
            var widget = getStructureWidget();
            widget.newRow();
            getRepositoryIdFromEstruturaWidget().reload();
            widget.currentRow.REPOSITORIO_ID = localStorage.getItem('repositorio');
            setCodeMirror(false);
            var issueField = getFieldIssuesStructureWidget();
            requestDataSourceEngine.getDataSource("/findIssuesRelatedToTask", []).then(function (response) {
                    if(response.dataset && response.dataset.findIssuesRelatedToTask){
                        var issueId = response.dataset.findIssuesRelatedToTask;
                        issueField.value(issueId);
                    }
                });
        }
    };
    
    this.structureActionDescSave = function(){
        var repository = localStorage.getItem('repositorio');
        var widget = getStructureWidget();
        var type = 'EDIT';
        widget.currentRow.TYPE_SAVE = type;
        var codeMirror = localStorageEngine.getLocalVar('CodeMirror');
        if(widget.isValid()){
            widget.currentRow.SCRIPT = codeMirror && codeMirror.doc.getValue() ? codeMirror.doc.getValue() : widget.currentRow.SCRIPT;
        StructureService.saveScript(widget.currentRow, type, null, repository).then(function(response){
            ScreenService.closePopup();
            widget.view();
            widget.currentRow = response;
            validateActions();
        }).catch(Utilities.showMessageErrorOfBackEnd);  
        } else{
            ScreenService.showMessage(ScreenService.i18n("Os campos com asterisco (*) devem ser preenchidos corretamente"));
        }
        
    };

    this.structureActionParse = function() {
        var widget = getStructureWidget();
        var codeMirror = localStorageEngine.getLocalVar('CodeMirror');
        var scripts = codeMirror && codeMirror.doc.getValue() ? codeMirror.doc.getValue() : widget.currentRow.SCRIPT;
        if(widget.isValid() && widget.currentRow.ISSUES){
            if (scripts.indexOf("DROP") >= 0 || scripts.indexOf("TRUNCATE") >= 0) {
                MessageService.showCustomDialogByCode('ALE_DANGEROUS_STATEMENT', null, {
                    type: 'CONFIRMATION',
                    actions: [{
                        "label": ScreenService.i18n("Prosseguir"),
                        "code": function () {
                            parseOrExec("PARSE");
                        }
                    }, {
                        "label": ScreenService.i18n("Cancelar"),
                        "code": function () {
                        }
                    }]
                });
            } else {
                parseOrExec("PARSE");
            }
        } else{
            ScreenService.showMessage(ScreenService.i18n("Os campos com asterisco (*) devem ser preenchidos corretamente"));
        }
        
    };

    this.structureActionParseScriptsWithErrors = function() {
        var widgetFix = getWidgetScriptFix();
        var widget = getStructureWidget();
        var logField   = getLogFromStructureWidget();
        var scripts = widgetFix.currentRow;
        widgetFix.currentRow = [];
        var singleScript = [];
        var fullScript = "";
        var repository = localStorage.getItem('repositorio');
        var type = (widget.currentRow.IDRESULTSCRIPT == "EE") ? "EXEC" : "PARSE";
        var issueField = getFieldIssuesStructureWidget();

        scriptsFieldsNames.forEach(function(script, index){
            singleScript[index] = {};
            var scriptName = script;
            var codeMirror = localStorageEngine.getLocalVar(scriptName);
            if(codeMirror){
                if(fullScript){
                    fullScript += "\n / \n" + codeMirror.doc.getValue();
                }else {
                    fullScript = codeMirror && codeMirror.doc.getValue() ? codeMirror.doc.getValue() : widget.currentRow[scriptName];
                }
                singleScript[index].SCRIPT = codeMirror && codeMirror.doc.getValue() ? codeMirror.doc.getValue() : widget.currentRow[scriptName];
                singleScript[index].ID = scriptName.substr(7);
            }

        });

        updateScript(widget, singleScript, type, logField, repository).then(function(resp){
            var widgetFix = getWidgetScriptFix();
            var fields = widgetFix.fields;
            widgetFix.fields = self.mtGenerationProjectIdFilter(fields);
            if (resp) {
                if (!widget.__is_new) {
                    ScreenService.closePopup();
                }
                widget.currentRow.LOG = logValue;
                validateActions();
                if (isFromErr) {
                    isFromErr = false;
                }
                widget.view();
                var message = "";
                var result = "";
                if (widget.currentRow.IDRESULTSCRIPT == "PE" || widget.currentRow.IDRESULTSCRIPT == "EE" || widget.currentRow.IDRESULTSCRIPT == "PD") {
                    logErrEdit = true;
                    logExec = false;
                    result = "ERROR";
                    message = "ERR_SCRIPT_RESULT";
                } else if (widget.currentRow.IDRESULTSCRIPT == "PS" || widget.currentRow.IDRESULTSCRIPT == "ES") {
                    if (widget.currentRow.IDRESULTSCRIPT == "PS") {
                        logExec = true;
                        logErrEdit = false;
                    }
                    result = "SUCCESS";
                    message = "SUC_SCRIPT_RESULT";
                }
                if (widget.currentRow.IDRESULTSCRIPT == "ES") {
                    self.validateWebSocketDirectory(resp);
                } else {
                    MessageService.showMessageByCode(message, null, {type: result});
                }
                self.structureActionEventViewLog();
                localStorageEngine.setLocalVar('CodeMirror', null);
            }
        });
        
    };

    this.structureActionExecuteScript = function() {
        var widget = getStructureWidget();
        var codeMirror = localStorageEngine.getLocalVar('CodeMirror');
        var scripts = codeMirror && codeMirror.doc.getValue() ? codeMirror.doc.getValue() : widget.currentRow.SCRIPT;
        if (scripts.indexOf("DROP") >= 0 || scripts.indexOf("TRUNCATE") >= 0) {
            MessageService.showCustomDialogByCode('ALE_DANGEROUS_STATEMENT', null, {
                type: 'CONFIRMATION',
                actions: [{
                    "label": ScreenService.i18n("Prosseguir"),
                    "code": function () {
                        parseOrExec("EXEC");
                    }
                }, {
                    "label": ScreenService.i18n("Cancelar"),
                    "code": function () {
                    }
                }]
            });
        } else {
            parseOrExec("EXEC");
        }
    };

    var parseOrExec = function (connType){
        setCodeMirror(false);
        var widget     = getStructureWidget();
        var logField   = getLogFromStructureWidget();
        var codeMirror = localStorageEngine.getLocalVar('CodeMirror');
        var scripts    = codeMirror && codeMirror.doc.getValue() ? codeMirror.doc.getValue() : widget.currentRow.SCRIPT;

        if(scripts.trim()) {
            widget.currentRow.SCRIPT = scripts.trim();
        } else {
            var message = ScreenService.i18n("Campo de Script vazio");
            ScreenService.showMessage(message);
            return false;
        }
        widget.edit();
        var invalidComments = StructureService.validateScriptsComments(scripts);
           if(!invalidComments){
               MessageService.showMessageByCode('ERR_INVALID_COMMENT');
               return false;
           }else {
               execAndSave(widget,scripts,logField, connType).then(function(resp) {
                   if (resp) {
                       if (!widget.__is_new) {
                           ScreenService.closePopup();
                       }
                       widget.currentRow.LOG = logValue;
                       validateActions();
                       if (isFromErr) {
                           isFromErr = false;
                       }
                       widget.view();
                       var message = "";
                       var result = "";
                       if (widget.currentRow.IDRESULTSCRIPT == "PE" || widget.currentRow.IDRESULTSCRIPT == "EE" || widget.currentRow.IDRESULTSCRIPT == "PD") {
                           logErrEdit = true;
                           result = "ERROR";
                           message = "ERR_SCRIPT_RESULT";
                       } else if (widget.currentRow.IDRESULTSCRIPT == "PS" || widget.currentRow.IDRESULTSCRIPT == "ES") {
                           if (widget.currentRow.IDRESULTSCRIPT == "PS") {
                               logExec = true;
                           }
                           result = "SUCCESS";
                           message = "SUC_SCRIPT_RESULT";
                       }
                       if (widget.currentRow.IDRESULTSCRIPT == "ES") {
                           self.validateWebSocketDirectory(resp);
                       } else {
                           MessageService.showMessageByCode(message, null, {type: result});
                       }
                       self.structureActionEventViewLog();
                       localStorageEngine.setLocalVar('CodeMirror', null);
                   }
               }).catch(function() {
                   ScreenService.restoreDefaultLoadingMessage();
               });
           }

    };

    this.structureViewExecAction = function(){
        ScreenService.closePopup();
        logExec = false;
        self.structureActionExecuteScript();
    };

    this.mtGenerationProjectIdFilter = function(array){
        return array.filter(function (field) {
            return !field.mtIsGeneration;
        });
    };

    this.structureActionErrEdit = function (){
        var widget = getStructureWidget();
        scriptsFieldsNames = [];
        var widgetFix = getWidgetScriptFix();
        var fields = widgetFix.fields;
        StructureService.getErrorsScripts(widget.currentRow).then(function(scriptIds){
            ScreenService.closePopup();
            scriptIds.forEach(function(scriptId){
                widgetFix.currentRow["SCRIPT_" + scriptId.ID] = scriptId.SCRIPT;
                var scriptField = self.getScriptFromErrorField(scriptId.ID, widgetFix);
                fields.push(scriptField);
                scriptsFieldsNames.push(scriptField.name);
                setCodeMirror(false, scriptField.name, widgetFix);
            });

            ScreenService.openPopup(widgetFix);

        });
    };

    var setCodeMirror = function(readOnly, campo, widget) {
        var codeMirror = campo ? campo : "CodeMirror";
        campo = campo ? campo : "SCRIPT";
        widget = widget ? widget :getStructureWidget();
        var prepareScriptField = function() {
            var element = $('#'+campo+':visible')[0];
            if(!element) return;
            var editor = CodeMirror.fromTextArea(element, {
                mode: "sql",
                extraKeys: {
                    "Ctrl-Space": "autocomplete",
                    "F11": function (cm) {
                        cm.setOption("fullScreen", !cm.getOption("fullScreen"));
                    },
                    "Esc": function (cm) {
                        if (cm.getOption("fullScreen")) cm.setOption("fullScreen", false);
                    }
                },
                lineNumbers: true,
                theme: "neat",
                placeholder: ScreenService.i18n("Utilize \"/\" como separador de comandos"),
                autoCloseBrackets: true,
                matchBrackets: true,
                readOnly: readOnly
            });
            localStorageEngine.setLocalVar(codeMirror, editor);
            widget.activate();
            clearInterval(interval);
        };
        var interval = setInterval(prepareScriptField, 10);
        var readOnlyLocalVar = readOnly ? 'TRUE' : 'FALSE';
        localStorage.setItem("isReadOnly", readOnlyLocalVar);
    };
    var updateScript = function(widget, singleScript, type, logField, repository){
        return StructureService.executeScriptsFromError(widget.currentRow, singleScript, type, logField).then(function(scriptArray) {
            return StructureService.getFullLog(widget.currentRow.DSDIRLOG).then(function(response) {
                return StructureService.updateScript(widget.currentRow, scriptArray, type, response, repository).then(function(response) {
                    ScreenService.restoreDefaultLoadingMessage();
                    widget.currentRow = response;
                        ScreenService.closePopup(0);
                        widget.activate();
                        localStorageEngine.setLocalVar("CodeMirror", null);
                        return response;
                });
            });
        });
    };
    var execAndSave = function(widget, scripts, logField, type){
        if(tempScript) {
            tempScript = null;
        }
        var repository = localStorage.getItem('repositorio');
        return StructureService.executeScript(widget.currentRow, scripts, type, logField).then(function(scriptArray) {
            logValue = widget.currentRow.LOG;
            return StructureService.getFullLog(widget.currentRow.DSDIRLOG).then(function(response){
                var codeMirror = localStorageEngine.getLocalVar('CodeMirror');
                widget.currentRow.SCRIPT =  widget.currentRow.SCRIPT = codeMirror && codeMirror.doc.getValue() ? codeMirror.doc.getValue() : widget.currentRow.SCRIPT;
                return StructureService.saveScript(widget.currentRow, type, response, repository, scriptArray).then(function(response) {
                    return MessageService.getMessageByCode('WAIT').then(function(messageCode) {
                        ScreenService.changeLoadingMessage(messageCode);
                        widget.currentRow = response;
                        localStorageEngine.getLocalVar('CodeMirror').doc.setValue(response.SCRIPT);
                        localStorageEngine.setLocalVar("CodeMirror", null);
                        return response;
                    });
                });
            });
        });
    };

    this.structureActionEventOpenUploadWidget = function() {
        var widget = getWidgetScriptUpload();
        ScreenService.openPopup(widget);
        var issueUploadField = getFieldIssuesUplodStructureWidget();
        
            requestDataSourceEngine.getDataSource("/findIssuesRelatedToTask", []).then(function (response) {
                    if(response.dataset && response.dataset.findIssuesRelatedToTask){
                        var issueId = response.dataset.findIssuesRelatedToTask;
                        issueUploadField.value(issueId);
                        Utilities.bugPaginationOnSelectMultiplyOnModeView(issueUploadField).then(function(){
                            widget.redraw();
                        });
                    }
                });
    };

    this.createFileWebSocket = function (dsDir, structure) {
        StructureService.downloadByWebSocket(localStorageEngine.getLocalVar('DSEMAIL'), dsDir, structure).then(function(url) {
            StructureService.removeTempFile(url);
            MessageService.showMessageByCode('SUC_DOWNLOADED', null, { type: 'SUCCESS' });
        }).catch(function (error) {
            if (error.connectionError) {
                MessageService.showCustomDialogByCode('WEB_SOCKET_NOT_FOUND', null, {
                    type: 'CONFIRMATION',
                    actions: [{
                        "label": ScreenService.i18n("Tentar Novamente"),
                        "code": function() {self.createFileWebSocket(dsDir, structure);}
                    }, {
                        "label": ScreenService.i18n("Download"),
                        "code": function() {StructureService.downloadScript(structure.ID);}
                    }]
                });
            } else {
                Utilities.showMessageBackEndTrue(error);
            }
        });
    };

    this.validateWebSocketDirectory = function (structure) {
        var repository = localStorage.getItem('repositorio');
        return StructureService.validateWebSocketDirectory(repository).then(function(dsDir){
            if(dsDir.DSDIRREPOSITORIO) {
                self.createFileWebSocket(dsDir.DSDIRREPOSITORIO, structure);
            } else {
                StructureService.downloadScript(structure.ID);
            }
        });
    };

    this.structureActionEventDownloadLog = function() {
        var widget = getStructureWidget();
        var currentRow = widget.currentRow;
        if(widget.currentRow.DSDIRLOG){
            widget.reload().then(function(){
                StructureService.downloadScriptLog(currentRow.ID, currentRow.DSDESESTRUTURA, currentRow.DSDIRLOG);
            });
        }else {
            ScreenService.showMessage(ScreenService.i18n("Log inexistente"));
        }

    };

    this.structureActionEventViewLog = function() {
        var widget = getStructureWidget();
        var logWidget = getViewLogWidget();
        if(widget.currentRow.DSDIRLOG){
            StructureService.getFullLog(widget.currentRow.DSDIRLOG).then(function(response){
                logWidget.currentRow.LOG = response;
                validateActions();
                ScreenService.openPopup(logWidget);
            });
        }else {
            ScreenService.showMessage(ScreenService.i18n("Log inexistente"));
        }

    };

    this.structureViewLogWidgetOnEnter = function() {
        var logWidget = getViewLogWidget();
        var log = logWidget.currentRow.LOG;
        var firstLine = ScreenService.i18n("Design Estrutura:");
        var pos = log.lastIndexOf(firstLine);
        var lastLog = log.substring(pos, log.length);
        logWidget.edit();

        var prepareScriptField = function() {
            var element = $('#LOG:visible')[0];
            if(!element) return;
            var fullHeight = element.scrollHeight;
            element.value = lastLog;
            var cutHeight = element.scrollHeight;
            element.value = log;
            var height = fullHeight - cutHeight;
            element.scrollTo(0, height);
            ScreenService.updateTemplate();
            clearInterval(interval);
        };
        var interval = setInterval(prepareScriptField, 10);
    };

    this.structureActionEventEdit = function() {
        var widget = getStructureWidget();
        var scriptField = getScriptFromStructureWidget();
        scriptField.readOnly = true;
        var issueField = getFieldIssuesStructureWidget();
        widget.edit();
        getRepositoryIdFromEstruturaWidget().reload();
        validateActions();
        if(widget.currentRow.IDRESULTSCRIPT != 'ES' && widget.currentRow.IDRESULTSCRIPT != 'PD'){
            setCodeMirror('nocursor');
        }else{
            setCodeMirror(false);
        }
    };

    var validateActions = function (){
        var widget = getStructureWidget();
        var action = getActionParsefromStructureWidget();
        var logAction = getActionBaixarLogfromStructureWidget();
        var scriptAction = getActionBaixarScriptfromStructureWidget();
        var errParse = getActionErrEditStructureWidget();
        var viewLog = getActionViewLogStructureWidget();
        var edit = getActionEditStructureWidget();
        var exec = getActionExecuteStructureWidget();
        var saveDesc = getActionSaveDescStructureWidget();
        var logErrEditAction = getActionErrEditViewLogWIdget();
        var logExecAction = getActionExecViewLogWIdget();
        var status = widget.currentRow.IDRESULTSCRIPT;
        var isActive = widget.currentRow.IDATIVO;
        var remove = getActionRemoveStructureWidget();
        var description = getDescriptionFromStructureWidget();
        var idativo = getIdAtivoFromStructureWidget();

        description.readOnly = (status == 'ES');
        idativo.readOnly = (status == 'ES');

        if(isActive == 'N') {
            edit.isVisible = false;
            action.isVisible = false;
            exec.isVisible = false;
            errParse.isVisible = false;
            scriptAction.isVisible = status == "ES";
        } else {
            if (logErrEdit) {
                logErrEditAction.isVisible = true;
                logExecAction.isVisible = false;
            } else if (logExec) {
                logExecAction.isVisible = true;
                logErrEditAction.isVisible = false;
            } else {
                logErrEditAction.isVisible = false;
                logExecAction.isVisible = false;
            }

            remove.isVisible = (status == 'PS' || status == 'PD' || status == 'PE');

            saveDesc.isVisible = (!isFromErr && widget.currentRow.ID);
            action.isVisible = (!widget.currentRow.ID || status == "PD" || (status == "PE" && isFromErr));
            exec.isVisible = (status == "PS" || (status == "EE" && isFromErr));
            errParse.isVisible = widget.mode != "edit" && (status == "PE" || status == "EE");
            scriptAction.isVisible = status == "ES";

            if (widget.currentRow.ID && widget.mode != "edit") {
                logAction.isVisible = true;
                viewLog.isVisible = true;
            } else {
                logAction.isVisible = false;
                viewLog.isVisible = false;
            }
        }
    };

    this.structureWidgetAfterMoveRow = function (){
        validateActions();
        var issueField = getFieldIssuesStructureWidget();
        Utilities.bugPaginationOnSelectMultiplyOnModeView(issueField);
    };

    this.repositoryWidgetOnEnter = function() {
        getRepositoryIdFromRepositoryWidget().reload();
    };

    this.structureActionEventDownloadScript = function() {
        var widget     = getStructureWidget();
        var currentRow = widget.currentRow;
        StructureService.downloadScript(currentRow.ID);
    };

    this.structureActionEventDelete = function() {
        MessageService.confirmMessageByCode('ALE_CONFIRM_REMOVE_REGISTER', null,
            {
                type: 'QUESTION',
                success: function() {
                    Utilities.genericDeleteEvent(getStructureWidget()).then(function() {
                        MessageService.successNotificationByCode('SUC_DELETED');
                    }).catch(Utilities.showMessageErrorOfBackEnd);
                },
                fail: function () {}
            });
    };

    this.structureActionEventUpload = function() {
        var widget = getWidgetScriptUpload();
        var logField   = getLogFromStructureWidget();
        var estrutura = getStructureWidget();
        var repository = localStorage.getItem('repositorio');
        if(widget.isValid() && widget.currentRow.DOCUMENT && widget.currentRow.ISSUES_UPLOAD){
            try{
                var script = Utilities.b64Decode(widget.currentRow.DOCUMENT[0].b64File.split(",").pop());
                StructureService.uploadScripts(script, logField, widget.currentRow, repository).then(function(){
                    ScreenService.closePopup(0);
                    MessageService.successNotificationByCode('SUC_UPLOADED');
                    estrutura.reload();
                }).catch(Utilities.showMessageErrorOfBackEnd);
            } catch (e) {
                MessageService.showMessageByCode('ERR_UPLOAD');
            }


        }else {
            ScreenService.showMessage(ScreenService.i18n("Os campos com asterisco (*) devem ser preenchidos corretamente"));
        }
    };
    

    /* --------------------------------------------- CONTAINER FUNCTIONS -------------------------------------------- */
    this.StructureContainerAfterinit = function () {
        var widget = (getStructureWidget());
            localStorageEngine.getLocalVar('loginPromise').then(function () {
                widget.activate();
                WebSocketClient.connectInServer();
            });
        };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('StructureController', StructureController);
});

// FILE: assets/js/Controller/Table.js
function TableController(ScreenService, ObjectsService, localStorageEngine,  templateManager, Utilities, ZhFilterPreferences){

    // ------------------------ GET WIDGETS ------------------------ \\

    var getTableWidget = function () {
        return templateManager.container.getWidget('TABLE');
    };
    var getTableTabWidget = function () {
        return templateManager.container.getWidget('TABLE_TAB');
    };
    var getColumnWidget = function () {
        return templateManager.container.getWidget('COLUMN');
    };
    var getConstraintWidget = function () {
        return templateManager.container.getWidget('CONSTRAINT');
    };
    var getTriggerWidget = function () {
        return templateManager.container.getWidget('TRIGGER');
    };
    var getConstraintColumnsWidget = function () {
        return templateManager.container.getWidget('CONSTRAINT_COLUMNS');
    };

    // ------------------------ Get Fields ------------------------ //

    var getFieldTabelaRelacionadaFromgetConstraintColumnsWidget = function(){
        return getConstraintColumnsWidget().getField('R_TABLE_NAME');
    };

    var getFieldColunaRelacionadaFromgetConstraintColumnsWidget = function(){
        return getConstraintColumnsWidget().getField('R_COLUMN_NAME');
    };

    // ------------------------ END ------------------------ //

    this.tableBasicBackAction = function(){
        ScreenService.closeSwipe();
    };

    this.tableWidgetAfterMoveRow = function(){
        var widget = getTableWidget();
        localStorageEngine.setLocalVar('table_name', widget.currentRow.TABLE_NAME);
        ObjectsService.loadTableObservations(widget.currentRow.TABLE_NAME).then(function(response){

            widget.currentRow.OBSERVATION = response;
        });
    };

    this.tableColumnWidgetAfterMoveRow = function(){
        var widget = getColumnWidget();
        ObjectsService.loadTableColumnObservations(widget.currentRow.COLUMN_NAME, widget.currentRow.TABLE_NAME).then(function(response){
            widget.currentRow.OBSERVATION = response;
        });
    };

    this.conColumnWidgetOnEnter = function(){
        getConstraintColumnsWidget().reload();
    };

    this.tableWidgetOnEnter = function() {
        ScreenService.showLoader();
        localStorageEngine.getLocalVar('loginPromise').then(function () {
            ScreenService.hideLoader();
            var widget = getTableWidget();
            ZhFilterPreferences.setOriginalFilter(widget.id, widget.currentRow);
            Utilities.applyDefaultFilterOnEnter(widget, true);
        });
    };
    this.tableBasicEditAction= function() {
        var widget = getTableWidget();
        localStorageEngine.setLocalVar('tableObservation', widget.currentRow.OBSERVATION);
        widget.edit();

    };

    this.constraintBasicEditAction= function() {
        var widget = getConstraintWidget();
        localStorageEngine.setLocalVar('constraintObservation', widget.currentRow.OBSERVATION);
        widget.edit();

    };

    this.constraintBasicCancelAction= function() {
        getConstraintWidget().currentRow.OBSERVATION =  localStorageEngine.getLocalVar('constraintObservation');
        getConstraintWidget().view();
    };

    this.constraintActionSaveObservation = function (){
        var widget = getConstraintWidget();
        ObjectsService.saveObjectsObservation(widget, 'C', widget.currentRow.CONSTRAINT_NAME).then(function(response){
            widget.view();
        }).catch(Utilities.showMessageErrorOfBackEnd);

    };

    this.tableBasicCancelAction= function() {
        getTableWidget().currentRow.OBSERVATION =  localStorageEngine.getLocalVar('tableObservation');
        getTableWidget().view();
    };

    this.columnBasicEditAction= function() {
        var widget = getColumnWidget();
        localStorageEngine.setLocalVar('columnObservation', widget.currentRow.OBSERVATION);
        widget.edit();
    };

    this.columnBasicCancelAction= function() {
        getColumnWidget().currentRow.OBSERVATION = localStorageEngine.getLocalVar('columnObservation');
        getColumnWidget().view();
    };


    this.columnWidgetOnEnter = function(){
        getColumnWidget().reload();
    };
    this.triggerWidgetOnEnter = function(){
        getTriggerWidget().reload();
    };

    this.triggerTabWidgetAfterMoveRow = function(){
        var widget = getTriggerWidget();
        var name = widget.currentRow.TRIGGER_NAME;
        var type = 'TRIGGER';
        ObjectsService.getObjectsBody(name, type, 'T').then(function(objectBody){
            widget.currentRow.BODY = objectBody.BODY;
            if(objectBody.OBSERVATION){
                widget.currentRow.OBSERVATION = objectBody.OBSERVATION;
            }
        });
    };

    this.constraintWidgetOnEnter = function(){
        getConstraintWidget().reload();
    };

    this.tableActionSaveObservation = function(){
        var widget = getTableWidget();
        ObjectsService.saveTableObservation(widget).then(function(response){
           widget.view();
        }).catch(Utilities.showMessageErrorOfBackEnd);
    };


    this.tableActionSaveColumnObservation = function(){
        var widget = getColumnWidget();
        ObjectsService.saveTableColumnObservation(widget).then(function(response){
           widget.view();
        }).catch(Utilities.showMessageErrorOfBackEnd);
    };

    this.constraintTabWidgetAfterMoveRow = function(){
        var widget = getConstraintWidget();
        var tipo =  widget.currentRow.CONSTRAINT_TYPE;
        var type =  "CONSTRAINT";
        var name = widget.currentRow.CONSTRAINT_NAME;
        var constraintWidget = getConstraintColumnsWidget();
        var tabelaR = getFieldTabelaRelacionadaFromgetConstraintColumnsWidget();
        var colunaR = getFieldColunaRelacionadaFromgetConstraintColumnsWidget();
        if(tipo === 'R') {
            constraintWidget.dataSourceFilter[1] =
                {
                    "name": "FK",
                    "operator": "=",
                    "value": "S"
                };
            tabelaR.showOnList = true;
            colunaR.showOnList = true;
        } else {
            constraintWidget.dataSourceFilter[1] = {
                "name": "FK",
                "operator": "=",
                "value": "N"
            };
            tabelaR.showOnList = false;
            colunaR.showOnList = false;
        }
        ObjectsService.getObjectsBody(name, type, 'C').then(function(objectBody){
            if(objectBody.OBSERVATION){
                widget.currentRow.OBSERVATION = objectBody.OBSERVATION;
            }
        });
    };

    this.tableWidgetFilterAfterApply = function (){
        Utilities.setFilterInit(getTableWidget());
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('TableController', TableController);
});

// FILE: assets/js/Controller/Trigger.js
function TriggerController(ScreenService, ObjectsService, localStorageEngine,  templateManager, Utilities) {

    // ------------------------ GET WIDGETS ------------------------ \\
    var getTriggerWidget = function () {
        return templateManager.container.getWidget('TRIGGER');
    };
    // --------------------------- END -------------------------------//

    this.triggerWidgetOnEnter = function() {
        ScreenService.showLoader();
        localStorageEngine.getLocalVar('loginPromise').then(function () {
            ScreenService.hideLoader();
            var widget = getTriggerWidget();
            //ZhFilterPreferences.setOriginalFilter(widget.id, widget.currentRow);
            Utilities.applyDefaultFilterOnEnter(widget, true);
        });
    };

    this.triggerBasicBackAction = function(){
        ScreenService.closeSwipe();
    };

    this.triggerBasicEditAction = function(){
        var widget = getTriggerWidget();
        localStorageEngine.setLocalVar('triggerObservation', widget.currentRow.OBSERVATION);
        widget.edit();
    };

    this.triggerBasicCancelAction = function(){
        getTriggerWidget().currentRow.OBSERVATION = localStorageEngine.getLocalVar('triggerObservation');
        getTriggerWidget().view();
    };

    this.triggerWidgetFilterAfterApply = function (){
        Utilities.setFilterInit(getTriggerWidget());
    };

    this.triggerWidgetAfterMoveRow = function (){
        var widget = getTriggerWidget();
        var name = widget.currentRow.TRIGGER_NAME;
        var type = 'TRIGGER';
        ObjectsService.getObjectsBody(name, type, 'T').then(function(objectBody){
            widget.currentRow.BODY = objectBody.BODY;
            if(objectBody.OBSERVATION){
                widget.currentRow.OBSERVATION = objectBody.OBSERVATION;
            }
        });
    };

    this.triggerActionSaveObservation = function (){
        var widget = getTriggerWidget();
        ObjectsService.saveObjectsObservation(widget, 'T', widget.currentRow.TRIGGER_NAME).then(function(response){
            widget.view();
        }).catch(Utilities.showMessageErrorOfBackEnd);

    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('TriggerController', TriggerController);
});

// FILE: assets/js/Controller/View.js
function ViewController(ScreenService, ObjectsService, localStorageEngine,  templateManager, Utilities) {

    // ------------------------ GET WIDGETS ------------------------ \\
    var getViewWidget = function () {
        return templateManager.container.getWidget('VIEW');
    };

    var getViewColumnsWidget = function () {
        return templateManager.container.getWidget('VIEW_COLUMNS');
    };
    // --------------------------- END -------------------------------//

    this.viewWidgetOnEnter = function() {
        ScreenService.showLoader();
        localStorageEngine.getLocalVar('loginPromise').then(function () {
            ScreenService.hideLoader();
            var widget = getViewWidget();
            Utilities.applyDefaultFilterOnEnter(widget, true);
        });
    };

    this.viewBasicBackAction = function(){
        ScreenService.closeSwipe();
    };

    this.viewWidgetFilterAfterApply = function (){
        Utilities.setFilterInit(getViewWidget());
    };

    this.viewBasicEditAction = function (){
        var widget = getViewWidget();
        localStorageEngine.setLocalVar('viewObservation', widget.currentRow.OBSERVATION);
        widget.edit();
    };

    this.viewBasicCancelAction = function (){
        getViewWidget().currentRow.OBSERVATION = localStorageEngine.getLocalVar('viewObservation');
        getViewWidget().view();
    };

    this.viewWidgetAfterMoveRow = function (){
        var widget = getViewWidget();
        var name = widget.currentRow.VIEW_NAME;
        var type = 'VIEW';
        ObjectsService.getObjectsBody(name, type, 'V').then(function(objectBody){
            widget.currentRow.BODY = objectBody.BODY;
            if(objectBody.OBSERVATION){
                widget.currentRow.OBSERVATION = objectBody.OBSERVATION;
            }
        });
    };

    this.columnWidgetAfterMoveRow = function (){
        var widget = getViewColumnsWidget();
        ObjectsService.loadTableColumnObservations(widget.currentRow.COLUMN_NAME, widget.currentRow.TABLE_NAME).then(function(response){
            widget.currentRow.OBSERVATION = response;
        });
    };


    this.viewActionSaveObservation = function (){
        var widget = getViewWidget();
        ObjectsService.saveObjectsObservation(widget, 'V', widget.currentRow.VIEW_NAME).then(function(response){
            widget.view();
        }).catch(Utilities.showMessageErrorOfBackEnd);

    };

    this.viewColumnWidgetOnEnter = function (){
        getViewColumnsWidget().reload();
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('ViewController', ViewController);
});